/*
 *  OpFlowTools.cpp
 *  
 *  Written by Patrick Heas.
 *  Copyright 2014 INRIA. All rights reserved.
 *
 */
#include <fstream>  
#include <iostream> 
#include "OpFlowTools.h"
using namespace cimg_library;
using namespace std;
 
extern "C"{
#include "cblas.h"
#include "clapack.h"
}

/**
 * product in fourier domain by convolution and low pass door
 */
CImg<double> productInFourier(const CImg<double> &im1,const CImg<double> &im2){
    return LowPassDoor(interpSinc(im1,2).get_mul(interpSinc(im2,2)),im1.width()/2.,2);
}


/**
 * interpolation 
 */
CImg<double> interpSinc(const CImg<double> &In,int factor){
    
    CImg<double> Out;
    if (factor>1){
        CImgList<double> fIn,fres;
        int dimx=In.width();
        int dimy=In.height();
        int halfNbx=dimx/2.;
        int halfNby=dimy/2.;
        
        //----Direct FFT
        fIn=In.get_FFT(false);
        fres.assign(2,dimx*factor,dimy*factor,1,1,0.);
        for(int i=0;i<halfNby;i++){
            for(int j=0;j<halfNbx;j++){
                fres[0](j,i)=fIn[0](j,i);
                fres[1](j,i)=fIn[1](j,i);
                if(j!=0){
                    fres[0](factor*dimx-j,i)=fIn[0](dimx-j,i);
                    fres[1](factor*dimx-j,i)=fIn[1](dimx-j,i);
                }
                if(i!=0){
                    fres[0](j,factor*dimy-i)=fIn[0](j,dimy-i);
                    fres[1](j,factor*dimy-i)=fIn[1](j,dimy-i);
                }
                if(i!=0&&j!=0){
                    fres[0](factor*dimx-j,factor*dimy-i)=fIn[0](dimx-j,dimy-i);
                    fres[1](factor*dimx-j,factor*dimy-i)=fIn[1](dimx-j,dimy-i);
                }
            }
        }
        /**/
        //attention pas complexe conjuge en N/2
        for(int i=0;i<halfNby;i++){
            fres[0](halfNbx,i)=fIn[0](halfNbx,i);
            fres[1](halfNbx,i)=fIn[1](halfNbx,i);
            if (i!=0){
                fres[0](halfNbx,factor*dimy-i)=fIn[0](halfNbx,dimy-i);
                fres[1](halfNbx,factor*dimy-i)=fIn[1](halfNbx,dimy-i);
            }
        }
        for(int j=0;j<halfNbx;j++){
            fres[0](j,halfNby)=fIn[0](j,halfNby);
            fres[1](j,halfNby)=fIn[1](j,halfNby);
            if (j!=0){
                fres[0](factor*dimx-j,halfNby)=fIn[0](dimx-j,halfNby);
                fres[1](factor*dimx-j,halfNby)=fIn[1](dimx-j,halfNby);
            }
        }
        fres[0](halfNbx,halfNby)=fIn[0](halfNbx,halfNby);
        fres[1](halfNbx,halfNby)=fIn[1](halfNbx,halfNby);
         /**/
        //----Inverse FFT
        Out= (fres.get_FFT(true))[0]*factor*factor;
    }
    else{Out=In;}
    return Out;
}
/**
 * Low pass filtering in Fourier domain and sub-sampling
 */
CImg<double> LowPassDoor(const CImg<double> &In,double cutOff){return LowPassDoor(In,cutOff,1);}

CImg<double> LowPassDoor(const CImg<double> &In,double cutOff,int factor){
    
     CImgList<double> fIn,fres;
    double maxK;
    int door;
	int dimx=In.width(),dimxF=dimx/factor;
	int dimy=In.height(),dimyF=dimy/factor;
    
    if (cutOff>dimxF/2){cutOff=dimxF/2.;}
    
	int halfNbx=dimxF/2.;
	int halfNby=dimyF/2.;
    CImg<double> res;
    if (factor==1 && cutOff>= halfNbx ){res=In;}
    else{
        
        fres.assign(2,dimxF,dimyF,1,1,0.);
        //----Direct FFT
        fIn=In.get_FFT(false);
        //door function application
        for(int i=0;i<halfNby;i++){
            for(int j=0;j<halfNbx;j++){
                maxK=max(sqrt((double)j*(double)j),sqrt((double)i*(double)i));
                (maxK>=cutOff+1)? door=0:door=1;
                if (door){
                    fres[0](j,i)=fIn[0](j,i)*door;
                    fres[1](j,i)=fIn[1](j,i)*door;
                    if(j!=0){
                        fres[0](dimxF-j,i)=fIn[0](dimx-j,i)*door;
                        fres[1](dimxF-j,i)=fIn[1](dimx-j,i)*door;
                    }
                    if(i!=0){
                        fres[0](j,dimyF-i)=fIn[0](j,dimy-i)*door;
                        fres[1](j,dimyF-i)=fIn[1](j,dimy-i)*door;
                    }
                    if(i!=0&&j!=0){
                        fres[0](dimxF-j,dimyF-i)=fIn[0](dimx-j,dimy-i)*door;
                        fres[1](dimxF-j,dimyF-i)=fIn[1](dimx-j,dimy-i)*door;
                    }
                }
            }
        }
        for(int i=0;i<halfNby;i++){
            maxK=max(sqrt(dimxF/2*dimxF/2),sqrt((double)i*(double)i));
            (maxK>=cutOff+1)? door=0:door=1;
            if (door){
                fres[0](dimxF/2,i)=2*fIn[0](dimxF/2,i)*door;
                fres[1](dimxF/2,i)=0*fIn[1](dimxF/2,i)*door;
                if (i>0){
                    fres[0](dimxF/2,dimyF-i)=2*fIn[0](dimxF/2,dimy-i)*door;
                    fres[1](dimxF/2,dimyF-i)=0*fIn[1](dimxF/2,dimy-i)*door;
                }
            }
        }
        
        for(int j=0;j<halfNbx;j++){
            maxK=max(sqrt(dimyF/2*dimyF/2),sqrt((double)j*(double)j));
            (maxK>=cutOff+1)? door=0:door=1;
            if (door){
                fres[0](j,dimyF/2)=2*fIn[0](j,dimyF/2)*door;
                fres[1](j,dimyF/2)=0*fIn[1](j,dimyF/2)*door;
                if (j>0){
                    fres[0](dimxF-j,dimyF/2)=2*fIn[0](dimx-j,dimyF/2)*door;
                    fres[1](dimxF-j,dimyF/2)=0*fIn[1](dimx-j,dimyF/2)*door;
                }
            }
        }
        maxK=sqrt(dimyF/2*dimyF/2);;// NB: same as max(sqrt(dimyF/2*dimyF/2),sqrt(dimxF/2*dimxF/2));
        (maxK>=cutOff+1)? door=0:door=1;
        if (door){
            fres[0](dimxF/2,dimyF/2)=2*fIn[0](dimxF/2,dimyF/2)*door;
            fres[1](dimxF/2,dimyF/2)=0*fIn[1](dimxF/2,dimyF/2)*door;
        }
        //----Inverse FFT
        res.assign((fres.get_FFT(true))[0]/(factor*factor));
        
    }
    return res;
    
}
/**
 * Low pass filtering and adjunction of a signal with spectral band corresponding to low pass
 */
CImg<double> LowPassDoorWithAddBandLimitedSignal(const CImg<double> &In,const CImg<double> &Im2Add){
    
    return LowPassDoor(In,Im2Add.width()/2)+interpSinc(Im2Add,In.width()/Im2Add.width());
    
    // a recrire avec frequence en N/2
    /*
    CImg<double> res; res.assign(In,false);
    if (Im2Add.width()>In.width()){cout<<"Error: lenght of added signal should be smaller than lenght of input signal"<<endl;}
    else{
        if (Im2Add.width()==In.width()){res+=Im2Add;}
        else{
            CImgList<double> fIn,fres,fAdd;
             int door;
            int dimx=In.width(),dimxF=Im2Add.width();
            int dimy=In.height(),dimyF=Im2Add.height();
            
            int halfNbx=dimx/2.;
            int halfNby=dimy/2.;
            int cutOff=dimxF/2.;
            int factor=dimx/dimxF;
            
            
            
            if (halfNbx!=cutOff){
                fres.assign(2,dimx,dimy,1,1,0.);
                //----Direct FFT
                fIn=In.get_FFT(false);
                fAdd=(Im2Add*(factor*factor)).get_FFT(false);
                
                //door function application
                for(int i=0;i<halfNby;i++){
                    for(int j=0;j<halfNbx;j++){
                        (max(j,i)>cutOff)? door=0:door=1;
                        if (door){
                            fres[0](j,i)=fIn[0](j,i)+fAdd[0](j,i);
                            fres[1](j,i)=fIn[1](j,i)+fAdd[1](j,i);
                            if(j!=0){
                                fres[0](dimx-j,i)=fIn[0](dimx-j,i); if(j!=cutOff) fres[0](dimx-j,i)+=fAdd[0](dimxF-j,i);
                                fres[1](dimx-j,i)=fIn[1](dimx-j,i); if(j!=cutOff) fres[1](dimx-j,i)+=fAdd[1](dimxF-j,i);
                            }
                            if(i!=0){
                                fres[0](j,dimy-i)=fIn[0](j,dimy-i); if(i!=cutOff) fres[0](j,dimy-i)+=fAdd[0](j,dimyF-i);
                                fres[1](j,dimy-i)=fIn[1](j,dimy-i); if(i!=cutOff) fres[1](j,dimy-i)+=fAdd[1](j,dimyF-i);
                            }
                            if(i!=0&&j!=0){
                                fres[0](dimx-j,dimy-i)=fIn[0](dimx-j,dimy-i); if(max(j,i)!=cutOff) fres[0](dimx-j,dimy-i)+=fAdd[0](dimxF-j,dimyF-i);
                                fres[1](dimx-j,dimy-i)=fIn[1](dimx-j,dimy-i); if(max(j,i)!=cutOff) fres[1](dimx-j,dimy-i)+=fAdd[1](dimxF-j,dimyF-i);
                            }
                        }
                    }
                }
                //----Inverse FFT
                res=(fres.get_FFT(true))[0];
            }
         }
    }
    return res;
    */
}


/**
 * curlImage
 */
CImg<double> curlImage(const CImg<double> &uv,
                       const CImg<double> &mask,
                       double curlbound,
                       double sigma){
  //--variables
  CImg<double> curl, palette;
  
  //-get jet palette
  palette.assign(jet_LUT256());  
  
  //-compute vort
  curl.assign(curlSpline(uv,sigma)); 
  //-and eventually apply mask
  if(!mask.is_empty()) {
    curl.mul(mask.get_channel(0).normalize());
  }
  
  //-display extrema and eventually cut values
  cout << "Curl values: [" << curl.min() << ";" << curl.max() << "] " << flush;
  if (curlbound){
    curl.cut(-curlbound,curlbound);
    cout << "restricted to [-" << curlbound << ";" << curlbound << "]" << flush;
  }
  cout << "." << endl;
  //-apply palette and return
  return curl.normalize(0,255).map(palette);
};

/**
 * curlPaint
 */
CImg<double> curlPaint(const CImg<double> &uv,
                       const CImg<double> &background,
                       const CImg<double> &mask,
                       double curlbound,
                       double darkshift,
                       double sigma){

  //--variables
  CImg<double> curl, bg;
  
  //-cut background to first channel
  bg.assign(background.get_channel(0));
  //-and shift dark regions towards gray to enhance display
  bg.normalize((int)(darkshift*255),255);
  
  //-get a colored image of curl
  curl.assign(curlImage(uv,mask,curlbound,sigma));  

  //-hard light
  return hardLight(curl,bg);
};

/**
 * normImage
 */
CImg<double> normImage(const CImg<double> &uv,
                       const CImg<double> &mask,
                       double normbound){
  //--variables
  CImg<double> norm, palette;
  
  //-get jet palette
  //palette.assign(jet_LUT256());
  //palette.assign(gradient2_LUT256(17,148,255, 255,236, 17));
  palette.assign(gradient3_LUT256(17,148,255, 255,236,17, 255, 17, 83));
  
  //-compute norm
  norm.assign(uv.get_shared_plane(0).get_sqr()+uv.get_shared_plane(1).get_sqr());
  norm.sqrt();
  //-and eventually apply mask
  if(!mask.is_empty()) {
    norm.mul(mask.get_channel(0).normalize());
  }
  
  //-display extrema and eventually cut values
  cout << "Norm values: [" << norm.min() << ";" << norm.max() << "] " << flush;
  if (normbound){
    norm.cut(0.,normbound);
    cout << "restricted to [0;" << normbound << "]" << flush;
  }
  cout << "." << endl;
  //-apply palette and return
  return norm.normalize(0,255).map(palette);
}; 
   
/**
 * normPaint
 */
CImg<double> normPaint(const CImg<double> &uv,
                       const CImg<double> &background,
                       const CImg<double> &mask,
                       double normbound,
                       double darkshift){
  
  //--variables
  CImg<double> norm, bg;

  //-cut background to first channel
  bg.assign(background.get_channel(0));
  //-and shift dark regions towards gray to enhance display
  bg.normalize((int)(darkshift*255),255);
  
  //-get colored norm
  norm.assign(normImage(uv,mask,normbound));
  
  //-hard light
  return hardLight(norm,bg);
};


void  fourierLerayProjector( CImgList<double> &fIm_u, CImgList<double> &fIm_v){
	
    //fIm_u[0].fill(1.);fIm_v[0].fill(1.);fIm_u[1].fill(1.);fIm_v[1].fill(1.);
	CImgList<double> fDIm_u(fIm_u,false);
	CImgList<double> fDIm_v(fIm_v,false);
	
	int dimx=fIm_u[0].width();
	int dimy=fIm_u[0].height();
	int halfNbx=dimx/2;
	int halfNby=dimy/2;
	double a11,a12,a22;
	double normK;
	double aux=4.*M_PI*M_PI;
	//----Projector	
    //NB: frequence N/2 inclue dans ces boucles (fequence sans complexe conjuge en N/2)
	for(int i=0;i<=halfNby;i++){
		for(int j=0;j<=halfNbx;j++){
			if (i==0 && j==0){normK=1;a11=0;a12=0;a22=0;}
			else{
				normK=aux*((double)j/(double)dimx*(double)j/(double)dimx+(double)i/(double)dimy*(double)i/(double)dimy);
				a11=(1.-(double)j/(double)dimx*(double)j/(double)dimx*aux/normK);
				a22=(1.-(double)i/(double)dimy*(double)i/(double)dimy*aux/normK);
				a12=-1.*(double)j/(double)dimx*(double)i/(double)dimy*aux/normK;
			}	
			fDIm_u[0](j,i) =fIm_u[0](j,i)*a11;
			fDIm_u[1](j,i) =fIm_u[1](j,i)*a11;
			fDIm_u[0](j,i)+=fIm_v[0](j,i)*a12;
			fDIm_u[1](j,i)+=fIm_v[1](j,i)*a12;
			fDIm_v[0](j,i) =fIm_u[0](j,i)*a12;
			fDIm_v[1](j,i) =fIm_u[1](j,i)*a12;
			fDIm_v[0](j,i)+=fIm_v[0](j,i)*a22;
			fDIm_v[1](j,i)+=fIm_v[1](j,i)*a22;
			if(j!=0){			
				fDIm_u[0](dimx-j,i) =fIm_u[0](dimx-j,i)*a11;
				fDIm_u[1](dimx-j,i) =fIm_u[1](dimx-j,i)*a11;
				fDIm_u[0](dimx-j,i)+=fIm_v[0](dimx-j,i)*a12*(-1.);
				fDIm_u[1](dimx-j,i)+=fIm_v[1](dimx-j,i)*a12*(-1.);
				fDIm_v[0](dimx-j,i) =fIm_u[0](dimx-j,i)*a12*(-1.);
				fDIm_v[1](dimx-j,i) =fIm_u[1](dimx-j,i)*a12*(-1.);
				fDIm_v[0](dimx-j,i)+=fIm_v[0](dimx-j,i)*a22;
				fDIm_v[1](dimx-j,i)+=fIm_v[1](dimx-j,i)*a22;
			}
			if(i!=0){			
				fDIm_u[0](j,dimy-i) =fIm_u[0](j,dimy-i)*a11;
				fDIm_u[1](j,dimy-i) =fIm_u[1](j,dimy-i)*a11;
				fDIm_u[0](j,dimy-i)+=fIm_v[0](j,dimy-i)*a12*(-1.);
				fDIm_u[1](j,dimy-i)+=fIm_v[1](j,dimy-i)*a12*(-1.);
				fDIm_v[0](j,dimy-i) =fIm_u[0](j,dimy-i)*a12*(-1.);
				fDIm_v[1](j,dimy-i) =fIm_u[1](j,dimy-i)*a12*(-1.);
				fDIm_v[0](j,dimy-i)+=fIm_v[0](j,dimy-i)*a22;
				fDIm_v[1](j,dimy-i)+=fIm_v[1](j,dimy-i)*a22;
			}
			if(i!=0&&j!=0){			
				fDIm_u[0](dimx-j,dimy-i) =fIm_u[0](dimx-j,dimy-i)*a11;
				fDIm_u[1](dimx-j,dimy-i) =fIm_u[1](dimx-j,dimy-i)*a11;
				fDIm_u[0](dimx-j,dimy-i)+=fIm_v[0](dimx-j,dimy-i)*a12;
				fDIm_u[1](dimx-j,dimy-i)+=fIm_v[1](dimx-j,dimy-i)*a12;
				fDIm_v[0](dimx-j,dimy-i) =fIm_u[0](dimx-j,dimy-i)*a12;
				fDIm_v[1](dimx-j,dimy-i) =fIm_u[1](dimx-j,dimy-i)*a12;
				fDIm_v[0](dimx-j,dimy-i)+=fIm_v[0](dimx-j,dimy-i)*a22;
				fDIm_v[1](dimx-j,dimy-i)+=fIm_v[1](dimx-j,dimy-i)*a22;
			}
		}	
		
	}
    

    
 	fIm_u=fDIm_u;
	fIm_v=fDIm_v;
	
}
void LerayProjector( CImg<double> &uv){

    CImg<double> uvout=uv;
    CImgList<double> fIm_u;
    CImgList<double> fIm_v;
	
    //----Direct FFT
    fIm_u=(uv.get_shared_plane(0)).get_FFT(false);
    fIm_v=(uv.get_shared_plane(1)).get_FFT(false);
    fourierLerayProjector(fIm_u,fIm_v);
    //----Inverse FFT
 	uv.get_shared_plane(0)=(fIm_u.get_FFT(true))[0];
	uv.get_shared_plane(1)=(fIm_v.get_FFT(true))[0];

}

void fourierBrownianSampling(CImg<double> &uv, double zeta, double minS, double maxS, bool divZero){
	
	//srand((unsigned)time(NULL));
	srand(0);  	
    cout<<"zeta:"<<zeta<<endl;
	int dimx=uv.width();
	int dimy=uv.height();
	int halfNbx=dimx/2.;
	int halfNby=dimy/2.;
	CImgList<double> fIm_u(uv.get_shared_plane(0),uv.get_shared_plane(0),false);
	CImgList<double> fIm_v(uv.get_shared_plane(1),uv.get_shared_plane(1),false);
	//conjugate symetric noise
	long int *auxNoise2=new long int[1];
	float gasdev(long *idum); 
	auxNoise2[0]=-(int)(((float)rand())/RAND_MAX*10000);
	fIm_u[0].fill(0);fIm_u[1].fill(0);
	fIm_v[0].fill(0);fIm_v[1].fill(0);
	double normK,normS,absKpowZeta;
	for(int i=0;i<halfNby;i++){
		for(int j=0;j<halfNbx;j++){
			if(i!=0||j!=0){
				normK=(double)j/(double)dimx*(double)j/(double)dimx+(double)i/(double)dimy*(double)i/(double)dimy;
				absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
				//normK*=4.*M_PI*M_PI;
				normS=1./sqrt(normK);
			}
			else{normS=-1;normK=-1;absKpowZeta=1;}
			if (normS>=minS && normS<=maxS){
			  if(i!=0&&j!=0){
				fIm_u[0](j,i) = gasdev(auxNoise2)*absKpowZeta;
				fIm_u[0](dimx-j,dimy-i) = fIm_u[0](j,i);
				fIm_u[1](dimx-j,dimy-i) = gasdev(auxNoise2)*absKpowZeta;
				fIm_u[1](j,i)=-1*fIm_u[1](dimx-j,dimy-i);
				fIm_v[0](j,i) = gasdev(auxNoise2)*absKpowZeta;
				fIm_v[0](dimx-j,dimy-i) = fIm_v[0](j,i);
				fIm_v[1](dimx-j,dimy-i) = gasdev(auxNoise2)*absKpowZeta;
				fIm_v[1](j,i)=-1*fIm_v[1](dimx-j,dimy-i);
			  
                  fIm_u[0](dimx-j,i) = gasdev(auxNoise2)*absKpowZeta;
                  fIm_u[0](j,dimy-i)=fIm_u[0](dimx-j,i);
                  fIm_v[1](dimx-j,i) = gasdev(auxNoise2)*absKpowZeta;
                  fIm_v[1](j,dimy-i)=-1*fIm_v[1](dimx-j,i);
                  fIm_v[0](dimx-j,i) = gasdev(auxNoise2)*absKpowZeta;
                  fIm_v[0](j,dimy-i)=fIm_v[0](dimx-j,i);
                  fIm_u[1](dimx-j,i) = gasdev(auxNoise2)*absKpowZeta;
                  fIm_u[1](j,dimy-i)=-1*fIm_u[1](dimx-j,i);
                  
                  fIm_u[0](j,dimy-i)= gasdev(auxNoise2)*absKpowZeta;
                  fIm_u[0](dimx-j,i)=fIm_u[0](j,dimy-i);
                  fIm_v[1](j,dimy-i)= gasdev(auxNoise2)*absKpowZeta;
                  fIm_v[1](dimx-j,i)=-1*fIm_v[1](j,dimy-i);
                  fIm_v[0](j,dimy-i)= gasdev(auxNoise2)*absKpowZeta;
                  fIm_v[0](dimx-j,i)=fIm_v[0](j,dimy-i);
                  fIm_u[1](j,dimy-i)= gasdev(auxNoise2)*absKpowZeta;
                  fIm_u[1](dimx-j,i)=-1*fIm_u[1](j,dimy-i);
                  
			  }
              if(j!=0&&i==0) {
                  fIm_u[0](dimx-j,0) = gasdev(auxNoise2)*absKpowZeta;fIm_u[0](j,0) = gasdev(auxNoise2)*absKpowZeta;
                  fIm_u[1](dimx-j,0) = gasdev(auxNoise2)*absKpowZeta;fIm_u[1](j,0) = gasdev(auxNoise2)*absKpowZeta;
                  fIm_v[0](dimx-j,0) = gasdev(auxNoise2)*absKpowZeta;fIm_v[0](j,0) = gasdev(auxNoise2)*absKpowZeta;
                  fIm_v[1](dimx-j,0) = gasdev(auxNoise2)*absKpowZeta;fIm_v[1](j,0) = gasdev(auxNoise2)*absKpowZeta;
             }
               if(j==0&&i!=0){ 
                  fIm_u[0](0,dimy-i)= gasdev(auxNoise2)*absKpowZeta;fIm_u[0](0,i)= gasdev(auxNoise2)*absKpowZeta;
                  fIm_u[1](0,dimy-i)= gasdev(auxNoise2)*absKpowZeta;fIm_u[1](0,i)= gasdev(auxNoise2)*absKpowZeta;
                  fIm_v[0](0,dimy-i)= gasdev(auxNoise2)*absKpowZeta;fIm_v[0](0,i)= gasdev(auxNoise2)*absKpowZeta;
                  fIm_v[1](0,dimy-i)= gasdev(auxNoise2)*absKpowZeta;fIm_v[1](0,i)= gasdev(auxNoise2)*absKpowZeta;
              }
              fIm_u[0](0,0)=0;
              fIm_u[1](0,0)=0;
              fIm_v[0](0,0)=0;
              fIm_v[1](0,0)=0;
              /**/
              //attention pas complexe conjuge en N/2
              for(int i=0;i<halfNby;i++){
                    normK=0.25+(double)i/(double)dimy*(double)i/(double)dimy;
                    absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
                    fIm_u[0](halfNbx,i)=gasdev(auxNoise2)*absKpowZeta;
                    fIm_u[1](halfNbx,i)=gasdev(auxNoise2)*absKpowZeta;
                    fIm_v[0](halfNbx,i)=gasdev(auxNoise2)*absKpowZeta;
                    fIm_v[1](halfNbx,i)=gasdev(auxNoise2)*absKpowZeta;
                    if (i!=0){ 
                        fIm_u[0](halfNbx,dimy-i)=gasdev(auxNoise2)*absKpowZeta;
                        fIm_u[1](halfNbx,dimy-i)=gasdev(auxNoise2)*absKpowZeta;
                        fIm_v[0](halfNbx,dimy-i)=gasdev(auxNoise2)*absKpowZeta;
                        fIm_v[1](halfNbx,dimy-i)=gasdev(auxNoise2)*absKpowZeta;
                    }
                }
                for(int j=0;j<halfNbx;j++){
                    normK=0.25+(double)j/(double)dimx*(double)j/(double)dimx;
                    absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
                    fIm_u[0](j,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                    fIm_u[1](j,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                    fIm_v[0](j,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                    fIm_v[1](j,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                    if (j!=0){ 
                        fIm_u[0](dimx-j,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                        fIm_u[1](dimx-j,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                        fIm_v[0](dimx-j,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                        fIm_v[1](dimx-j,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                    }
                }
                normK=0.25+0.25;
                absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
                fIm_u[0](halfNbx,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                fIm_u[1](halfNbx,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                fIm_v[0](halfNbx,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                fIm_v[1](halfNbx,halfNby)=gasdev(auxNoise2)*absKpowZeta;
                /**/
			}
		}
	}

    //----Leray Div zero projection
	if (divZero){fourierLerayProjector(fIm_u,fIm_v);}
	//----Inverse FFT
	uv.get_shared_plane(0)=(fIm_u.get_FFT(true))[0]*sqrt(dimx*dimy);
	uv.get_shared_plane(1)=(fIm_v.get_FFT(true))[0]*sqrt(dimx*dimy);
}

CImg<double>  fracDerivOrInt(const CImg<double> &Im,double zeta){
 
    CImg<double>  res;
    if (Im.depth()==1){res=fracDerivOrInt_1Band(Im,zeta);}
    else{res=fracDerivOrInt(Im,zeta,false);}
    return res;

}
CImg<double>  fracDerivOrInt(const CImg<double> &Im,double zeta, bool LerayProj){
	
	
	  CImgList<double> fIm_u;
	  CImgList<double> fIm_v;
	
	  //----Direct FFT
	  fIm_u=(Im.get_slice(0)).get_FFT(false);
	  fIm_v=(Im.get_slice(1)).get_FFT(false);
      CImgList<double> fDIm_u(fIm_u);
 	  CImgList<double> fDIm_v(fIm_v);
      fDIm_u[0].fill(0); fDIm_u[1].fill(0); 
      fDIm_v[0].fill(0); fDIm_v[1].fill(0);
	
	  int dimx=Im.width();
	  int dimy=Im.height();
	  int halfNbx=dimx/2;
	  int halfNby=dimy/2;
	  double absKpowZeta;
	  double normK;
	
	  //----Fractional derivatiave/intergation
	
	  for(int i=0;i<halfNby;i++){
		for(int j=0;j<halfNbx;j++){
			if (i==0 && j==0){absKpowZeta=1;}
			else{
				normK=(double)j/(double)dimx*(double)j/(double)dimx+(double)i/(double)dimy*(double)i/(double)dimy;
				absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
			}	
			fDIm_u[0](j,i)=fIm_u[0](j,i)*absKpowZeta;
			fDIm_u[1](j,i)=fIm_u[1](j,i)*absKpowZeta;
			fDIm_v[0](j,i)=fIm_v[0](j,i)*absKpowZeta;
			fDIm_v[1](j,i)=fIm_v[1](j,i)*absKpowZeta;
			if(j!=0){			
				fDIm_u[0](dimx-j,i)=fIm_u[0](dimx-j,i)*absKpowZeta;
				fDIm_u[1](dimx-j,i)=fIm_u[1](dimx-j,i)*absKpowZeta;
				fDIm_v[0](dimx-j,i)=fIm_v[0](dimx-j,i)*absKpowZeta;
				fDIm_v[1](dimx-j,i)=fIm_v[1](dimx-j,i)*absKpowZeta;
			}
			if(i!=0){			
				fDIm_u[0](j,dimy-i)=fIm_u[0](j,dimy-i)*absKpowZeta;
				fDIm_u[1](j,dimy-i)=fIm_u[1](j,dimy-i)*absKpowZeta;
				fDIm_v[0](j,dimy-i)=fIm_v[0](j,dimy-i)*absKpowZeta;
				fDIm_v[1](j,dimy-i)=fIm_v[1](j,dimy-i)*absKpowZeta;
			}
			if(i!=0&&j!=0){			
				fDIm_u[0](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*absKpowZeta;
				fDIm_u[1](dimx-j,dimy-i)=fIm_u[1](dimx-j,dimy-i)*absKpowZeta;
				fDIm_v[0](dimx-j,dimy-i)=fIm_v[0](dimx-j,dimy-i)*absKpowZeta;
				fDIm_v[1](dimx-j,dimy-i)=fIm_v[1](dimx-j,dimy-i)*absKpowZeta;
			}
		}	
		  
	  }
    /**/
    //attention pas complexe conjuge en N/2
	  for(int i=0;i<halfNby;i++){
		normK=0.25+(double)i/(double)dimy*(double)i/(double)dimy;
		absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
        fDIm_u[0](halfNbx,i)=fIm_u[0](halfNbx,i)*absKpowZeta;
		fDIm_u[1](halfNbx,i)=fIm_u[1](halfNbx,i)*absKpowZeta;
		fDIm_v[0](halfNbx,i)=fIm_v[0](halfNbx,i)*absKpowZeta;
		fDIm_v[1](halfNbx,i)=fIm_v[1](halfNbx,i)*absKpowZeta;
        if (i!=0){ 
            fDIm_u[0](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*absKpowZeta;
            fDIm_u[1](halfNbx,dimy-i)=fIm_u[1](halfNbx,dimy-i)*absKpowZeta;
            fDIm_v[0](halfNbx,dimy-i)=fIm_v[0](halfNbx,dimy-i)*absKpowZeta;
            fDIm_v[1](halfNbx,dimy-i)=fIm_v[1](halfNbx,dimy-i)*absKpowZeta;
        }
	  }
	  for(int j=0;j<halfNbx;j++){
			normK=0.25+(double)j/(double)dimx*(double)j/(double)dimx;
			absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
		    fDIm_u[0](j,halfNby)=fIm_u[0](j,halfNby)*absKpowZeta;
			fDIm_u[1](j,halfNby)=fIm_u[1](j,halfNby)*absKpowZeta;
			fDIm_v[0](j,halfNby)=fIm_v[0](j,halfNby)*absKpowZeta;
			fDIm_v[1](j,halfNby)=fIm_v[1](j,halfNby)*absKpowZeta;
          if (j!=0){ 
              fDIm_u[0](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*absKpowZeta;
              fDIm_u[1](dimx-j,halfNby)=fIm_u[1](dimx-j,halfNby)*absKpowZeta;
              fDIm_v[0](dimx-j,halfNby)=fIm_v[0](dimx-j,halfNby)*absKpowZeta;
              fDIm_v[1](dimx-j,halfNby)=fIm_v[1](dimx-j,halfNby)*absKpowZeta;
          }
      }
	  normK=0.25+0.25;
	  absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
	  fDIm_u[0](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*absKpowZeta;
	  fDIm_u[1](halfNbx,halfNby)=fIm_u[1](halfNbx,halfNby)*absKpowZeta;
	  fDIm_v[0](halfNbx,halfNby)=fIm_v[0](halfNbx,halfNby)*absKpowZeta;
	  fDIm_v[1](halfNbx,halfNby)=fIm_v[1](halfNbx,halfNby)*absKpowZeta;
     /**/ 
    if (LerayProj){fourierLerayProjector(fDIm_u,fDIm_v);}	
	//----Inverse FFT
	CImg<double> ImOut(Im,"xyzc",0.);
	ImOut.get_shared_plane(0)=(fDIm_u.get_FFT(true))[0];
	ImOut.get_shared_plane(1)=(fDIm_v.get_FFT(true))[0];
	return ImOut;
	
}

CImg<double>  fracIntOrDeriv(const CImg<double> &Im,double zeta){
	return fracDerivOrInt(Im,-1*zeta);
}
CImg<double>  fracIntOrDeriv(const CImg<double> &Im,double zeta,bool LerayProj){
	return fracDerivOrInt(Im,-1*zeta,LerayProj);
}
CImg<double>  fracDerivOrInt_CurlAdjoint(const CImg<double> &Im,double zeta){
    
    return fracDerivOrInt_CurlAdjoint(Im,zeta,false);
}

CImg<double>  fracDerivOrInt_CurlAdjoint(const CImg<double> &Im,double zeta,bool twoParts){
	
	
	CImgList<double> fIm_u;
	CImgList<double> fIm_v;
	
	//----Direct FFT
	fIm_u=(Im.get_slice(0)).get_FFT(false);
	fIm_v=(Im.get_slice(1)).get_FFT(false);
	CImgList<double> fDIm_u(fIm_u);
	CImgList<double> fDIm_v(fIm_v);
	
	int dimx=Im.width();
	int dimy=Im.height();
	int halfNbx=dimx/2;
	int halfNby=dimy/2;
	double absKpowZeta;
	double normK;
	
	//----Fractional derivatiave/intergation
	if(zeta!=0){
		for(int i=0;i<halfNby;i++){
			for(int j=0;j<halfNbx;j++){
				if (i==0 && j==0){absKpowZeta=0;}
				else{
					normK=(double)j/(double)dimx*(double)j/(double)dimx+(double)i/(double)dimy*(double)i/(double)dimy;
					absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
				}	
				fDIm_u[0](j,i)=fIm_u[0](j,i)*absKpowZeta;
				fDIm_u[1](j,i)=fIm_u[1](j,i)*absKpowZeta;
				fDIm_v[0](j,i)=fIm_v[0](j,i)*absKpowZeta;
				fDIm_v[1](j,i)=fIm_v[1](j,i)*absKpowZeta;
				if(j!=0){			
					fDIm_u[0](dimx-j,i)=fIm_u[0](dimx-j,i)*absKpowZeta;
					fDIm_u[1](dimx-j,i)=fIm_u[1](dimx-j,i)*absKpowZeta;
					fDIm_v[0](dimx-j,i)=fIm_v[0](dimx-j,i)*absKpowZeta;
					fDIm_v[1](dimx-j,i)=fIm_v[1](dimx-j,i)*absKpowZeta;
				}
				if(i!=0){			
					fDIm_u[0](j,dimy-i)=fIm_u[0](j,dimy-i)*absKpowZeta;
					fDIm_u[1](j,dimy-i)=fIm_u[1](j,dimy-i)*absKpowZeta;
					fDIm_v[0](j,dimy-i)=fIm_v[0](j,dimy-i)*absKpowZeta;
					fDIm_v[1](j,dimy-i)=fIm_v[1](j,dimy-i)*absKpowZeta;
				}
				if(i!=0&&j!=0){			
					fDIm_u[0](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*absKpowZeta;
					fDIm_u[1](dimx-j,dimy-i)=fIm_u[1](dimx-j,dimy-i)*absKpowZeta;
					fDIm_v[0](dimx-j,dimy-i)=fIm_v[0](dimx-j,dimy-i)*absKpowZeta;
					fDIm_v[1](dimx-j,dimy-i)=fIm_v[1](dimx-j,dimy-i)*absKpowZeta;
				}
			}	
			
		}
        /**/
        //attention pas complexe conjuge en N/2
        for(int i=0;i<halfNby;i++){
            normK=0.25+(double)i/(double)dimy*(double)i/(double)dimy;
            absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
            fDIm_u[0](halfNbx,i)=fIm_u[0](halfNbx,i)*absKpowZeta;
            fDIm_u[1](halfNbx,i)=fIm_u[1](halfNbx,i)*absKpowZeta;
            fDIm_v[0](halfNbx,i)=fIm_v[0](halfNbx,i)*absKpowZeta;
            fDIm_v[1](halfNbx,i)=fIm_v[1](halfNbx,i)*absKpowZeta;
            if (i!=0){ 
                fDIm_u[0](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*absKpowZeta;
                fDIm_u[1](halfNbx,dimy-i)=fIm_u[1](halfNbx,dimy-i)*absKpowZeta;
                fDIm_v[0](halfNbx,dimy-i)=fIm_v[0](halfNbx,dimy-i)*absKpowZeta;
                fDIm_v[1](halfNbx,dimy-i)=fIm_v[1](halfNbx,dimy-i)*absKpowZeta;
            }
        }
        for(int j=0;j<halfNbx;j++){
			normK=0.25+(double)j/(double)dimx*(double)j/(double)dimx;
			absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
		    fDIm_u[0](j,halfNby)=fIm_u[0](j,halfNby)*absKpowZeta;
			fDIm_u[1](j,halfNby)=fIm_u[1](j,halfNby)*absKpowZeta;
			fDIm_v[0](j,halfNby)=fIm_v[0](j,halfNby)*absKpowZeta;
			fDIm_v[1](j,halfNby)=fIm_v[1](j,halfNby)*absKpowZeta;
            if (j!=0){ 
                fDIm_u[0](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*absKpowZeta;
                fDIm_u[1](dimx-j,halfNby)=fIm_u[1](dimx-j,halfNby)*absKpowZeta;
                fDIm_v[0](dimx-j,halfNby)=fIm_v[0](dimx-j,halfNby)*absKpowZeta;
                fDIm_v[1](dimx-j,halfNby)=fIm_v[1](dimx-j,halfNby)*absKpowZeta;
            }
        }
        normK=0.25+0.25;
        absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
        fDIm_u[0](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*absKpowZeta;
        fDIm_u[1](halfNbx,halfNby)=fIm_u[1](halfNbx,halfNby)*absKpowZeta;
        fDIm_v[0](halfNbx,halfNby)=fIm_v[0](halfNbx,halfNby)*absKpowZeta;
        fDIm_v[1](halfNbx,halfNby)=fIm_v[1](halfNbx,halfNby)*absKpowZeta;
        /**/ 
		
		fIm_u=fDIm_u;
		fIm_v=fDIm_v;
		
	}
    //---- curl  adjoint operator 
	double factorWav_u, factorWav_v;
	for(int i=0;i<halfNby;i++){
		for(int j=0;j<halfNbx;j++){
            
			if (i!=0){factorWav_u=-2.*M_PI*(double)i/(double)dimy; }
			else {factorWav_u=0;}
			if (j!=0){factorWav_v=2.*M_PI*(double)j/(double)dimx;}
			else {factorWav_v=0;}
            
            fDIm_u[0](j,i)=-1*fIm_u[1](j,i)*factorWav_u;
            fDIm_u[1](j,i)=fIm_u[0](j,i)*factorWav_u;
            fDIm_v[0](j,i)=-1*fIm_v[1](j,i)*factorWav_v;
            fDIm_v[1](j,i)=fIm_v[0](j,i)*factorWav_v;
            
			if(j!=0&& j!=halfNbx){			
				fDIm_u[0](dimx-j,i)=-1*fIm_u[1](dimx-j,i)*+1*factorWav_u;
				fDIm_u[1](dimx-j,i)=fIm_u[0](dimx-j,i)*+1*factorWav_u;
				fDIm_v[0](dimx-j,i)=-1*fIm_v[1](dimx-j,i)*-1*factorWav_v;
				fDIm_v[1](dimx-j,i)=fIm_v[0](dimx-j,i)*-1*factorWav_v;
			}
			if(i!=0 && i!=halfNby){			
				fDIm_u[0](j,dimy-i)=-1*fIm_u[1](j,dimy-i)*-1*factorWav_u;
				fDIm_u[1](j,dimy-i)=fIm_u[0](j,dimy-i)*-1*factorWav_u;
				fDIm_v[0](j,dimy-i)=-1*fIm_v[1](j,dimy-i)*+1*factorWav_v;
				fDIm_v[1](j,dimy-i)=fIm_v[0](j,dimy-i)*+1*factorWav_v;
			}
			if(i!=0&&i!=halfNby&&j!=0&& j!=halfNbx){			
				fDIm_u[0](dimx-j,dimy-i)=-1*fIm_u[1](dimx-j,dimy-i)*-1*factorWav_u;
				fDIm_u[1](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*-1*factorWav_u;
				fDIm_v[0](dimx-j,dimy-i)=-1*fIm_v[1](dimx-j,dimy-i)*-1*factorWav_v;
				fDIm_v[1](dimx-j,dimy-i)=fIm_v[0](dimx-j,dimy-i)*-1*factorWav_v;
			}
		}
	}	    
    //attention pas complexe conjuge en N/2
    for(int i=0;i<halfNby;i++){
        if (i!=0){
            factorWav_u=-2.*M_PI*(double)i/(double)dimy;
        }
        else {factorWav_u=0;}
        factorWav_v=2.*M_PI*(double)halfNbx/(double)dimx;
        
        fDIm_u[0](halfNbx,i)=-1*fIm_u[1](halfNbx,i)*factorWav_u;
        fDIm_u[1](halfNbx,i)=fIm_u[0](halfNbx,i)*factorWav_u;
        fDIm_v[0](halfNbx,i)=-1*fIm_u[1](halfNbx,i)*factorWav_v;
        fDIm_v[1](halfNbx,i)=fIm_u[0](halfNbx,i)*factorWav_v;	
        if (i!=0){ 
            fDIm_u[0](halfNbx,dimy-i)=-1*fIm_u[1](halfNbx,dimy-i)*-1*factorWav_u;
            fDIm_u[1](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*-1*factorWav_u;
            fDIm_v[0](halfNbx,dimy-i)=-1*fIm_u[1](halfNbx,dimy-i)*factorWav_v;
            fDIm_v[1](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*factorWav_v;	
        }
    }
    for(int j=0;j<halfNbx;j++){
        factorWav_u=-2.*M_PI*(double)halfNby/(double)dimy;
        if (j!=0){
            factorWav_v=2.*M_PI*(double)j/(double)dimx;
        }
        else {factorWav_v=0;}
        
        fDIm_u[0](j,halfNby)=-1*fIm_u[1](j,halfNby)*factorWav_u;
        fDIm_u[1](j,halfNby)=fIm_u[0](j,halfNby)*factorWav_u;
        fDIm_v[0](j,halfNby)=-1*fIm_u[1](j,halfNby)*factorWav_v;
        fDIm_v[1](j,halfNby)=fIm_u[0](j,halfNby)*factorWav_v;	
        if (j!=0){ 
            fDIm_u[0](dimx-j,halfNby)=-1*fIm_u[1](dimx-j,halfNby)*factorWav_u;
            fDIm_u[1](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*factorWav_u;
            fDIm_v[0](dimx-j,halfNby)=-1*fIm_u[1](dimx-j,halfNby)*-1*factorWav_v;
            fDIm_v[1](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*-1*factorWav_v;	
        }       
    }
    factorWav_u=-2.*M_PI*(double)halfNby/(double)dimy;
    factorWav_v=2.*M_PI*(double)halfNbx/(double)dimx;
    fDIm_u[0](halfNbx,halfNby)=-1*fIm_u[1](halfNbx,halfNby)*factorWav_u;
    fDIm_u[1](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*factorWav_u;
    fDIm_v[0](halfNbx,halfNby)=-1*fIm_u[1](halfNbx,halfNby)*factorWav_v;
    fDIm_v[1](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*factorWav_v;	
    
    /**/ 
	//----Inverse FFT
    
	CImg<double> ImOut;
    if (!twoParts){
        ImOut.assign(dimx,dimy,1,1,0.);
        fDIm_u[0]+=fDIm_v[0];fDIm_u[1]+=fDIm_v[1];
        ImOut=(fDIm_u.get_FFT(true))[0];
    }
    else{
        ImOut.assign(dimx,dimy,2,1,0.);
        ImOut.get_shared_plane(0)=(fDIm_u.get_FFT(true))[0];
        ImOut.get_shared_plane(1)=(fDIm_v.get_FFT(true))[0];
    }
	return ImOut;
	
}

CImg<double>  fracDerivOrInt_Curl(const CImg<double> &Im,double zeta){
	
	CImgList<double> fIm_u;
	//----Direct FFT
	fIm_u=Im.get_FFT(false);
	CImgList<double> fDIm_u(fIm_u);fDIm_u[0].fill(0.);fDIm_u[1].fill(0.);
	CImgList<double> fDIm_v(fIm_u);fDIm_v[0].fill(0.);fDIm_v[1].fill(0.);
	
	int dimx=Im.width();
	int dimy=Im.height();
	int halfNbx=dimx/2;
	int halfNby=dimy/2;
	double absKpowZeta;
	double normK;
	
	//----Fractional derivatiave/intergation
	if(zeta!=0){
		for(int i=0;i<halfNby;i++){
			for(int j=0;j<halfNbx;j++){
				if (i==0 && j==0){absKpowZeta=0;}
				else{
					normK=(double)j/(double)dimx*(double)j/(double)dimx+(double)i/(double)dimy*(double)i/(double)dimy;
					absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
				}	
				fDIm_u[0](j,i)=fIm_u[0](j,i)*absKpowZeta;
				fDIm_u[1](j,i)=fIm_u[1](j,i)*absKpowZeta;
				if(j!=0){			
					fDIm_u[0](dimx-j,i)=fIm_u[0](dimx-j,i)*absKpowZeta;
					fDIm_u[1](dimx-j,i)=fIm_u[1](dimx-j,i)*absKpowZeta;
				}
				if(i!=0){			
					fDIm_u[0](j,dimy-i)=fIm_u[0](j,dimy-i)*absKpowZeta;
					fDIm_u[1](j,dimy-i)=fIm_u[1](j,dimy-i)*absKpowZeta;
				}
				if(i!=0&&j!=0){			
					fDIm_u[0](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*absKpowZeta;
					fDIm_u[1](dimx-j,dimy-i)=fIm_u[1](dimx-j,dimy-i)*absKpowZeta;
				}
			}	
			
		}
        //attention pas complexe conjuge en N/2
        for(int i=0;i<halfNby;i++){
            normK=0.25+(double)i/(double)dimy*(double)i/(double)dimy;
            absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
            fDIm_u[0](halfNbx,i)=fIm_u[0](halfNbx,i)*absKpowZeta;
            fDIm_u[1](halfNbx,i)=fIm_u[1](halfNbx,i)*absKpowZeta;
            if (i!=0){ 
                fDIm_u[0](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*absKpowZeta;
                fDIm_u[1](halfNbx,dimy-i)=fIm_u[1](halfNbx,dimy-i)*absKpowZeta;
            }
        }
        for(int j=0;j<halfNbx;j++){
			normK=0.25+(double)j/(double)dimx*(double)j/(double)dimx;
			absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
		    fDIm_u[0](j,halfNby)=fIm_u[0](j,halfNby)*absKpowZeta;
			fDIm_u[1](j,halfNby)=fIm_u[1](j,halfNby)*absKpowZeta;
            if (j!=0){ 
                fDIm_u[0](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*absKpowZeta;
                fDIm_u[1](dimx-j,halfNby)=fIm_u[1](dimx-j,halfNby)*absKpowZeta;
            }
        }
        normK=0.25+0.25;
        absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
        fDIm_u[0](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*absKpowZeta;
        fDIm_u[1](halfNbx,halfNby)=fIm_u[1](halfNbx,halfNby)*absKpowZeta;
        /**/ 
		
		fIm_u=fDIm_u;
		
	}
	//---- Curl operator 
 	double factorWav_u, factorWav_v;
	for(int i=0;i<halfNby;i++){
			for(int j=0;j<halfNbx;j++){
				
				if (i!=0){
                    factorWav_u=2*M_PI*(double)i/(double)dimy;
                }
				else {factorWav_u=0;}
				if (j!=0){
                    factorWav_v=-2*M_PI*(double)j/(double)dimx;
                }	
				else {factorWav_v=0;}
				
                fDIm_u[0](j,i)=-1*fIm_u[1](j,i)*factorWav_u;
                fDIm_u[1](j,i)=fIm_u[0](j,i)*factorWav_u;
                fDIm_v[0](j,i)=-1*fIm_u[1](j,i)*factorWav_v;
                fDIm_v[1](j,i)=fIm_u[0](j,i)*factorWav_v;	
                
				if(j!=0 && j!=halfNbx){			
					fDIm_u[0](dimx-j,i)=-1*fIm_u[1](dimx-j,i)*+1*factorWav_u;
					fDIm_u[1](dimx-j,i)=fIm_u[0](dimx-j,i)*+1*factorWav_u;
					fDIm_v[0](dimx-j,i)=-1*fIm_u[1](dimx-j,i)*-1*factorWav_v;
					fDIm_v[1](dimx-j,i)=fIm_u[0](dimx-j,i)*-1*factorWav_v;
				}
				if(i!=0&& i!=halfNby){			
					fDIm_u[0](j,dimy-i)=-1*fIm_u[1](j,dimy-i)*-1*factorWav_u;
					fDIm_u[1](j,dimy-i)=fIm_u[0](j,dimy-i)*-1*factorWav_u;
					fDIm_v[0](j,dimy-i)=-1*fIm_u[1](j,dimy-i)*+1*factorWav_v;
					fDIm_v[1](j,dimy-i)=fIm_u[0](j,dimy-i)*+1*factorWav_v;
				}
				if(i!=0&&j!=0&& i!=halfNby&& j!=halfNbx){			
					fDIm_u[0](dimx-j,dimy-i)=-1*fIm_u[1](dimx-j,dimy-i)*-1*factorWav_u;
					fDIm_u[1](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*-1*factorWav_u;
					fDIm_v[0](dimx-j,dimy-i)=-1*fIm_u[1](dimx-j,dimy-i)*-1*factorWav_v;
					fDIm_v[1](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*-1*factorWav_v;
				}
			}
	}
    //attention pas complexe conjuge en N/2
    for(int i=0;i<halfNby;i++){
        if (i!=0){
            factorWav_u=2*M_PI*(double)i/(double)dimy;
        }
        else {factorWav_u=0;}
        factorWav_v=-2*M_PI*(double)halfNbx/(double)dimx;
    
        fDIm_u[0](halfNbx,i)=-1*fIm_u[1](halfNbx,i)*factorWav_u;
        fDIm_u[1](halfNbx,i)=fIm_u[0](halfNbx,i)*factorWav_u;
        fDIm_v[0](halfNbx,i)=-1*fIm_u[1](halfNbx,i)*factorWav_v;
        fDIm_v[1](halfNbx,i)=fIm_u[0](halfNbx,i)*factorWav_v;	
        if (i!=0){ 
            fDIm_u[0](halfNbx,dimy-i)=-1*fIm_u[1](halfNbx,dimy-i)*-1*factorWav_u;
            fDIm_u[1](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*-1*factorWav_u;
            fDIm_v[0](halfNbx,dimy-i)=-1*fIm_u[1](halfNbx,dimy-i)*factorWav_v;
            fDIm_v[1](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*factorWav_v;	
        }
    }
    for(int j=0;j<halfNbx;j++){
        factorWav_u=2*M_PI*(double)halfNby/(double)dimy;
        if (j!=0){
            factorWav_v=-2*M_PI*(double)j/(double)dimx;
        }
        else {factorWav_v=0;}

        fDIm_u[0](j,halfNby)=-1*fIm_u[1](j,halfNby)*factorWav_u;
        fDIm_u[1](j,halfNby)=fIm_u[0](j,halfNby)*factorWav_u;
        fDIm_v[0](j,halfNby)=-1*fIm_u[1](j,halfNby)*factorWav_v;
        fDIm_v[1](j,halfNby)=fIm_u[0](j,halfNby)*factorWav_v;	
        if (j!=0){ 
            fDIm_u[0](dimx-j,halfNby)=-1*fIm_u[1](dimx-j,halfNby)*factorWav_u;
            fDIm_u[1](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*factorWav_u;
            fDIm_v[0](dimx-j,halfNby)=-1*fIm_u[1](dimx-j,halfNby)*-1*factorWav_v;
            fDIm_v[1](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*-1*factorWav_v;	
        }       
    }
    factorWav_u=2*M_PI*(double)halfNby/(double)dimy;
    factorWav_v=-2*M_PI*(double)halfNbx/(double)dimx;
    fDIm_u[0](halfNbx,halfNby)=-1*fIm_u[1](halfNbx,halfNby)*factorWav_u;
    fDIm_u[1](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*factorWav_u;
    fDIm_v[0](halfNbx,halfNby)=-1*fIm_u[1](halfNbx,halfNby)*factorWav_v;
    fDIm_v[1](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*factorWav_v;	
    /**/ 
	//----Inverse FFT
	CImg<double> ImOut(Im.width(),Im.height(),2,1,0.);
	ImOut.get_shared_plane(0)=(fDIm_u.get_FFT(true))[0];
	ImOut.get_shared_plane(1)=(fDIm_v.get_FFT(true))[0];
	return ImOut;
	
}
CImg<double>  fracDerivOrInt_CurlAndGrad(const CImg<double> &Im,double zeta){
	
	CImgList<double> fIm_u;

	//----Direct FFT
	fIm_u=Im.get_FFT(false);
    CImgList<double> fIm_uAux(fIm_u,0);
	CImgList<double> fDIm_u(fIm_u,0);fDIm_u[0].fill(0.);fDIm_u[1].fill(0.);
	CImgList<double> fDIm_v(fIm_u,0);fDIm_v[0].fill(0.);fDIm_v[1].fill(0.);
	CImgList<double> fGIm_u(fIm_u,0);fGIm_u[0].fill(0.);fGIm_u[1].fill(0.);
	CImgList<double> fGIm_v(fIm_u,0);fGIm_v[0].fill(0.);fGIm_v[1].fill(0.);
	
	int dimx=Im.width();
	int dimy=Im.height();
	int halfNbx=dimx/2;
	int halfNby=dimy/2;
	double absKpowZeta;
	double normK;
	
	//----Fractional derivatiave/intergation
	if(zeta!=0){
		for(int i=0;i<halfNby;i++){
			for(int j=0;j<halfNbx;j++){
				if (i==0 && j==0){absKpowZeta=0;}
				else{
					normK=(double)j/(double)dimx*(double)j/(double)dimx+(double)i/(double)dimy*(double)i/(double)dimy;
					absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
				}
				fDIm_u[0](j,i)=fIm_u[0](j,i)*absKpowZeta;
				fDIm_u[1](j,i)=fIm_u[1](j,i)*absKpowZeta;
				if(j!=0){
					fDIm_u[0](dimx-j,i)=fIm_u[0](dimx-j,i)*absKpowZeta;
					fDIm_u[1](dimx-j,i)=fIm_u[1](dimx-j,i)*absKpowZeta;
				}
				if(i!=0){
					fDIm_u[0](j,dimy-i)=fIm_u[0](j,dimy-i)*absKpowZeta;
					fDIm_u[1](j,dimy-i)=fIm_u[1](j,dimy-i)*absKpowZeta;
				}
				if(i!=0&&j!=0){
					fDIm_u[0](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*absKpowZeta;
					fDIm_u[1](dimx-j,dimy-i)=fIm_u[1](dimx-j,dimy-i)*absKpowZeta;
				}
			}
			
		}
        //attention pas complexe conjuge en N/2
        for(int i=0;i<halfNby;i++){
            normK=0.25+(double)i/(double)dimy*(double)i/(double)dimy;
            absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
            fDIm_u[0](halfNbx,i)=fIm_u[0](halfNbx,i)*absKpowZeta;
            fDIm_u[1](halfNbx,i)=fIm_u[1](halfNbx,i)*absKpowZeta;
            if (i!=0){
                fDIm_u[0](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*absKpowZeta;
                fDIm_u[1](halfNbx,dimy-i)=fIm_u[1](halfNbx,dimy-i)*absKpowZeta;
            }
        }
        for(int j=0;j<halfNbx;j++){
			normK=0.25+(double)j/(double)dimx*(double)j/(double)dimx;
			absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
		    fDIm_u[0](j,halfNby)=fIm_u[0](j,halfNby)*absKpowZeta;
			fDIm_u[1](j,halfNby)=fIm_u[1](j,halfNby)*absKpowZeta;
            if (j!=0){
                fDIm_u[0](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*absKpowZeta;
                fDIm_u[1](dimx-j,halfNby)=fIm_u[1](dimx-j,halfNby)*absKpowZeta;
            }
        }
        normK=0.25+0.25;
        absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
        fDIm_u[0](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*absKpowZeta;
        fDIm_u[1](halfNbx,halfNby)=fIm_u[1](halfNbx,halfNby)*absKpowZeta;
        /**/
		
		fIm_u=fDIm_u;
		
	}
	//---- Curl or Grad operators
 	double factorWav_u, factorWav_v,factorWav_Gu,factorWav_Gv;
	for(int i=0;i<halfNby;i++){
        for(int j=0;j<halfNbx;j++){
            
            if (i!=0){
                factorWav_u=2*M_PI*(double)i/(double)dimy;
            }
            else {factorWav_u=0;}
            if (j!=0){
                factorWav_v=-2*M_PI*(double)j/(double)dimx;
            }
            else {factorWav_v=0;}
            factorWav_Gu=factorWav_v*-1.;
            factorWav_Gv=factorWav_u;
            
            
            fDIm_u[0](j,i)=-1*fIm_u[1](j,i)*factorWav_u;        fGIm_u[0](j,i)=-1*fIm_uAux[1](j,i)*factorWav_Gu;
            fDIm_u[1](j,i)=fIm_u[0](j,i)*factorWav_u;           fGIm_u[1](j,i)=fIm_uAux[0](j,i)*factorWav_Gu;
            fDIm_v[0](j,i)=-1*fIm_u[1](j,i)*factorWav_v;        fGIm_v[0](j,i)=-1*fIm_uAux[1](j,i)*factorWav_Gv;
            fDIm_v[1](j,i)=fIm_u[0](j,i)*factorWav_v;           fGIm_v[1](j,i)=fIm_uAux[0](j,i)*factorWav_Gv;
            
            if(j!=0 && j!=halfNbx){
                fDIm_u[0](dimx-j,i)=-1*fIm_u[1](dimx-j,i)*+1*factorWav_u;   fGIm_u[0](dimx-j,i)=-1*fIm_uAux[1](dimx-j,i)*-1*factorWav_Gu;
                fDIm_u[1](dimx-j,i)=fIm_u[0](dimx-j,i)*+1*factorWav_u;      fGIm_u[1](dimx-j,i)=fIm_uAux[0](dimx-j,i)*-1*factorWav_Gu;
                fDIm_v[0](dimx-j,i)=-1*fIm_u[1](dimx-j,i)*-1*factorWav_v;   fGIm_v[0](dimx-j,i)=-1*fIm_uAux[1](dimx-j,i)*+1*factorWav_Gv;
                fDIm_v[1](dimx-j,i)=fIm_u[0](dimx-j,i)*-1*factorWav_v;      fGIm_v[1](dimx-j,i)=fIm_uAux[0](dimx-j,i)*+1*factorWav_Gv;
            }
            if(i!=0&& i!=halfNby){
                fDIm_u[0](j,dimy-i)=-1*fIm_u[1](j,dimy-i)*-1*factorWav_u;   fGIm_u[0](j,dimy-i)=-1*fIm_uAux[1](j,dimy-i)*+1*factorWav_Gu;
                fDIm_u[1](j,dimy-i)=fIm_u[0](j,dimy-i)*-1*factorWav_u;      fGIm_u[1](j,dimy-i)=fIm_uAux[0](j,dimy-i)*+1*factorWav_Gu;
                fDIm_v[0](j,dimy-i)=-1*fIm_u[1](j,dimy-i)*+1*factorWav_v;   fGIm_v[0](j,dimy-i)=-1*fIm_uAux[1](j,dimy-i)*-1*factorWav_Gv;
                fDIm_v[1](j,dimy-i)=fIm_u[0](j,dimy-i)*+1*factorWav_v;      fGIm_v[1](j,dimy-i)=fIm_uAux[0](j,dimy-i)*-1*factorWav_Gv;
            }
            if(i!=0&&j!=0&& i!=halfNby&& j!=halfNbx){
                fDIm_u[0](dimx-j,dimy-i)=-1*fIm_u[1](dimx-j,dimy-i)*-1*factorWav_u;     fGIm_u[0](dimx-j,dimy-i)=-1*fIm_uAux[1](dimx-j,dimy-i)*-1*factorWav_Gu;
                fDIm_u[1](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*-1*factorWav_u;        fGIm_u[1](dimx-j,dimy-i)=fIm_uAux[0](dimx-j,dimy-i)*-1*factorWav_Gu;
                fDIm_v[0](dimx-j,dimy-i)=-1*fIm_u[1](dimx-j,dimy-i)*-1*factorWav_v;     fGIm_v[0](dimx-j,dimy-i)=-1*fIm_uAux[1](dimx-j,dimy-i)*-1*factorWav_Gv;
                fDIm_v[1](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*-1*factorWav_v;        fGIm_v[1](dimx-j,dimy-i)=fIm_uAux[0](dimx-j,dimy-i)*-1*factorWav_Gv;
            }
        }
	}
    //attention pas complexe conjuge en N/2
    for(int i=0;i<halfNby;i++){
        if (i!=0){
            factorWav_u=2*M_PI*(double)i/(double)dimy;
        }
        else {factorWav_u=0;}
        factorWav_v=-2*M_PI*(double)halfNbx/(double)dimx;
        factorWav_Gu=factorWav_v*-1.;
        factorWav_Gv=factorWav_u;
       
        fDIm_u[0](halfNbx,i)=-1*fIm_u[1](halfNbx,i)*factorWav_u;    fGIm_u[0](halfNbx,i)=-1*fIm_uAux[1](halfNbx,i)*factorWav_Gu;
        fDIm_u[1](halfNbx,i)=fIm_u[0](halfNbx,i)*factorWav_u;       fGIm_u[1](halfNbx,i)=fIm_uAux[0](halfNbx,i)*factorWav_Gu;
        fDIm_v[0](halfNbx,i)=-1*fIm_u[1](halfNbx,i)*factorWav_v;    fGIm_v[0](halfNbx,i)=-1*fIm_uAux[1](halfNbx,i)*factorWav_Gv;
        fDIm_v[1](halfNbx,i)=fIm_u[0](halfNbx,i)*factorWav_v;       fGIm_v[1](halfNbx,i)=fIm_uAux[0](halfNbx,i)*factorWav_Gv;
        if (i!=0){
            fDIm_u[0](halfNbx,dimy-i)=-1*fIm_u[1](halfNbx,dimy-i)*-1*factorWav_u;  fGIm_u[0](halfNbx,dimy-i)=-1*fIm_uAux[1](halfNbx,dimy-i)*factorWav_Gu;
            fDIm_u[1](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*-1*factorWav_u;     fGIm_u[1](halfNbx,dimy-i)=fIm_uAux[0](halfNbx,dimy-i)*factorWav_Gu;
            fDIm_v[0](halfNbx,dimy-i)=-1*fIm_u[1](halfNbx,dimy-i)*factorWav_v;  fGIm_v[0](halfNbx,dimy-i)=-1*fIm_uAux[1](halfNbx,dimy-i)*-1*factorWav_Gv;
            fDIm_v[1](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*factorWav_v;     fGIm_v[1](halfNbx,dimy-i)=fIm_uAux[0](halfNbx,dimy-i)*-1*factorWav_Gv;
        }
    }
    for(int j=0;j<halfNbx;j++){
        factorWav_u=2*M_PI*(double)halfNby/(double)dimy;
        if (j!=0){
            factorWav_v=-2*M_PI*(double)j/(double)dimx;
        }
        else {factorWav_v=0;}
        factorWav_Gu=factorWav_v*-1.;
        factorWav_Gv=factorWav_u;
       
        fDIm_u[0](j,halfNby)=-1*fIm_u[1](j,halfNby)*factorWav_u;        fGIm_u[0](j,halfNby)=-1*fIm_uAux[1](j,halfNby)*factorWav_Gu;
        fDIm_u[1](j,halfNby)=fIm_u[0](j,halfNby)*factorWav_u;           fGIm_u[1](j,halfNby)=fIm_uAux[0](j,halfNby)*factorWav_Gu;
        fDIm_v[0](j,halfNby)=-1*fIm_u[1](j,halfNby)*factorWav_v;        fGIm_v[0](j,halfNby)=-1*fIm_uAux[1](j,halfNby)*factorWav_Gv;
        fDIm_v[1](j,halfNby)=fIm_u[0](j,halfNby)*factorWav_v;           fGIm_v[1](j,halfNby)=fIm_uAux[0](j,halfNby)*factorWav_Gv;
        if (j!=0){
            fDIm_u[0](dimx-j,halfNby)=-1*fIm_u[1](dimx-j,halfNby)*factorWav_u;  fGIm_u[0](dimx-j,halfNby)=-1*fIm_uAux[1](dimx-j,halfNby)*-1*factorWav_Gu;
            fDIm_u[1](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*factorWav_u;     fGIm_u[1](dimx-j,halfNby)=fIm_uAux[0](dimx-j,halfNby)*-1*factorWav_Gu;
            fDIm_v[0](dimx-j,halfNby)=-1*fIm_u[1](dimx-j,halfNby)*-1*factorWav_v;  fGIm_v[0](dimx-j,halfNby)=-1*fIm_uAux[1](dimx-j,halfNby)*factorWav_Gv;
            fDIm_v[1](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*-1*factorWav_v;     fGIm_v[1](dimx-j,halfNby)=fIm_uAux[0](dimx-j,halfNby)*factorWav_Gv;
        }
    }
    factorWav_u=2*M_PI*(double)halfNby/(double)dimy;
    factorWav_v=-2*M_PI*(double)halfNbx/(double)dimx;
    factorWav_Gu=factorWav_v*-1.;
    factorWav_Gv=factorWav_u;
    fDIm_u[0](halfNbx,halfNby)=-1*fIm_u[1](halfNbx,halfNby)*factorWav_u;    fGIm_u[0](halfNbx,halfNby)=-1*fIm_uAux[1](halfNbx,halfNby)*factorWav_Gu;
    fDIm_u[1](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*factorWav_u;       fGIm_u[1](halfNbx,halfNby)=fIm_uAux[0](halfNbx,halfNby)*factorWav_Gu;
    fDIm_v[0](halfNbx,halfNby)=-1*fIm_u[1](halfNbx,halfNby)*factorWav_v;    fGIm_v[0](halfNbx,halfNby)=-1*fIm_uAux[1](halfNbx,halfNby)*factorWav_Gv;
    fDIm_v[1](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*factorWav_v;       fGIm_v[1](halfNbx,halfNby)=fIm_uAux[0](halfNbx,halfNby)*factorWav_Gv;
    /**/
	//----Inverse FFT
	CImg<double> ImOut(Im.width(),Im.height(),4,1,0.);
	ImOut.get_shared_plane(0)=(fDIm_u.get_FFT(true))[0];
	ImOut.get_shared_plane(1)=(fDIm_v.get_FFT(true))[0];
	ImOut.get_shared_plane(2)=(fGIm_u.get_FFT(true))[0];
	ImOut.get_shared_plane(3)=(fGIm_v.get_FFT(true))[0];
      
	return ImOut;
	
}
CImg<double>  gradFourier(const CImg<double> &Im){
	
	CImgList<double> fIm_u;
    
	//----Direct FFT
	fIm_u=Im.get_FFT(false);
    CImgList<double> fIm_uAux(fIm_u,0);
	CImgList<double> fGIm_u(fIm_u,0);fGIm_u[0].fill(0.);fGIm_u[1].fill(0.);
	CImgList<double> fGIm_v(fIm_u,0);fGIm_v[0].fill(0.);fGIm_v[1].fill(0.);
	
	int dimx=Im.width();
	int dimy=Im.height();
	int halfNbx=dimx/2;
	int halfNby=dimy/2;
	
	//---- Grad operators
 	double factorWav_u, factorWav_v,factorWav_Gu,factorWav_Gv;
	for(int i=0;i<halfNby;i++){
        for(int j=0;j<halfNbx;j++){
            
            if (i!=0){
                factorWav_u=2*M_PI*(double)i/(double)dimy;
            }
            else {factorWav_u=0;}
            if (j!=0){
                factorWav_v=-2*M_PI*(double)j/(double)dimx;
            }
            else {factorWav_v=0;}
            factorWav_Gu=factorWav_v*-1.;
            factorWav_Gv=factorWav_u;
            
            
            fGIm_u[0](j,i)=-1*fIm_uAux[1](j,i)*factorWav_Gu;
            fGIm_u[1](j,i)=fIm_uAux[0](j,i)*factorWav_Gu;
            fGIm_v[0](j,i)=-1*fIm_uAux[1](j,i)*factorWav_Gv;
            fGIm_v[1](j,i)=fIm_uAux[0](j,i)*factorWav_Gv;
            
            if(j!=0 && j!=halfNbx){
                fGIm_u[0](dimx-j,i)=-1*fIm_uAux[1](dimx-j,i)*-1*factorWav_Gu;
                fGIm_u[1](dimx-j,i)=fIm_uAux[0](dimx-j,i)*-1*factorWav_Gu;
                fGIm_v[0](dimx-j,i)=-1*fIm_uAux[1](dimx-j,i)*+1*factorWav_Gv;
                fGIm_v[1](dimx-j,i)=fIm_uAux[0](dimx-j,i)*+1*factorWav_Gv;
            }
            if(i!=0&& i!=halfNby){
                fGIm_u[0](j,dimy-i)=-1*fIm_uAux[1](j,dimy-i)*+1*factorWav_Gu;
                fGIm_u[1](j,dimy-i)=fIm_uAux[0](j,dimy-i)*+1*factorWav_Gu;
                fGIm_v[0](j,dimy-i)=-1*fIm_uAux[1](j,dimy-i)*-1*factorWav_Gv;
                fGIm_v[1](j,dimy-i)=fIm_uAux[0](j,dimy-i)*-1*factorWav_Gv;
            }
            if(i!=0&&j!=0&& i!=halfNby&& j!=halfNbx){
                fGIm_u[0](dimx-j,dimy-i)=-1*fIm_uAux[1](dimx-j,dimy-i)*-1*factorWav_Gu;
                fGIm_u[1](dimx-j,dimy-i)=fIm_uAux[0](dimx-j,dimy-i)*-1*factorWav_Gu;
                fGIm_v[0](dimx-j,dimy-i)=-1*fIm_uAux[1](dimx-j,dimy-i)*-1*factorWav_Gv;
                fGIm_v[1](dimx-j,dimy-i)=fIm_uAux[0](dimx-j,dimy-i)*-1*factorWav_Gv;
            }
        }
	}
    //attention pas complexe conjuge en N/2
    for(int i=0;i<halfNby;i++){
        if (i!=0){
            factorWav_u=2*M_PI*(double)i/(double)dimy;
        }
        else {factorWav_u=0;}
        factorWav_v=-2*M_PI*(double)halfNbx/(double)dimx;
        factorWav_Gu=factorWav_v*-1.;
        factorWav_Gv=factorWav_u;
        
        fGIm_u[0](halfNbx,i)=-1*fIm_uAux[1](halfNbx,i)*factorWav_Gu;
        fGIm_u[1](halfNbx,i)=fIm_uAux[0](halfNbx,i)*factorWav_Gu;
        fGIm_v[0](halfNbx,i)=-1*fIm_uAux[1](halfNbx,i)*factorWav_Gv;
        fGIm_v[1](halfNbx,i)=fIm_uAux[0](halfNbx,i)*factorWav_Gv;
        if (i!=0){
            fGIm_u[0](halfNbx,dimy-i)=-1*fIm_uAux[1](halfNbx,dimy-i)*factorWav_Gu;
            fGIm_u[1](halfNbx,dimy-i)=fIm_uAux[0](halfNbx,dimy-i)*factorWav_Gu;
            fGIm_v[0](halfNbx,dimy-i)=-1*fIm_uAux[1](halfNbx,dimy-i)*-1*factorWav_Gv;
            fGIm_v[1](halfNbx,dimy-i)=fIm_uAux[0](halfNbx,dimy-i)*-1*factorWav_Gv;
        }
    }
    for(int j=0;j<halfNbx;j++){
        factorWav_u=2*M_PI*(double)halfNby/(double)dimy;
        if (j!=0){
            factorWav_v=-2*M_PI*(double)j/(double)dimx;
        }
        else {factorWav_v=0;}
        factorWav_Gu=factorWav_v*-1.;
        factorWav_Gv=factorWav_u;
        
        fGIm_u[0](j,halfNby)=-1*fIm_uAux[1](j,halfNby)*factorWav_Gu;
        fGIm_u[1](j,halfNby)=fIm_uAux[0](j,halfNby)*factorWav_Gu;
        fGIm_v[0](j,halfNby)=-1*fIm_uAux[1](j,halfNby)*factorWav_Gv;
        fGIm_v[1](j,halfNby)=fIm_uAux[0](j,halfNby)*factorWav_Gv;
        if (j!=0){
            fGIm_u[0](dimx-j,halfNby)=-1*fIm_uAux[1](dimx-j,halfNby)*-1*factorWav_Gu;
            fGIm_u[1](dimx-j,halfNby)=fIm_uAux[0](dimx-j,halfNby)*-1*factorWav_Gu;
            fGIm_v[0](dimx-j,halfNby)=-1*fIm_uAux[1](dimx-j,halfNby)*factorWav_Gv;
            fGIm_v[1](dimx-j,halfNby)=fIm_uAux[0](dimx-j,halfNby)*factorWav_Gv;
        }
    }
    factorWav_u=2*M_PI*(double)halfNby/(double)dimy;
    factorWav_v=-2*M_PI*(double)halfNbx/(double)dimx;
    factorWav_Gu=factorWav_v*-1.;
    factorWav_Gv=factorWav_u;
    fGIm_u[0](halfNbx,halfNby)=-1*fIm_uAux[1](halfNbx,halfNby)*factorWav_Gu;
    fGIm_u[1](halfNbx,halfNby)=fIm_uAux[0](halfNbx,halfNby)*factorWav_Gu;
    fGIm_v[0](halfNbx,halfNby)=-1*fIm_uAux[1](halfNbx,halfNby)*factorWav_Gv;
    fGIm_v[1](halfNbx,halfNby)=fIm_uAux[0](halfNbx,halfNby)*factorWav_Gv;
    /**/
	//----Inverse FFT
	CImg<double> ImOut(Im.width(),Im.height(),2,1,0.);
	ImOut.get_shared_plane(0)=(fGIm_u.get_FFT(true))[0];
	ImOut.get_shared_plane(1)=(fGIm_v.get_FFT(true))[0];
    
	return ImOut;
	
}

CImg<double>  fracDerivOrInt_1Band(const CImg<double> &Im,double zeta){
	
	CImgList<double> fIm_u;
	//----Direct FFT
	fIm_u=Im.get_FFT(false);
	CImgList<double> fDIm_u(fIm_u);fDIm_u[0].fill(0.);fDIm_u[1].fill(0.);
	
	int dimx=Im.width();
	int dimy=Im.height();
	int halfNbx=dimx/2;
	int halfNby=dimy/2;
	double absKpowZeta;
	double normK;
	
	//----Fractional derivatiave/intergation
	if(zeta!=0){
		for(int i=0;i<halfNby;i++){
			for(int j=0;j<halfNbx;j++){
				if (i==0 && j==0){absKpowZeta=0;}
				else{
					normK=(double)j/(double)dimx*(double)j/(double)dimx+(double)i/(double)dimy*(double)i/(double)dimy;
					absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
				}	
				fDIm_u[0](j,i)=fIm_u[0](j,i)*absKpowZeta;
				fDIm_u[1](j,i)=fIm_u[1](j,i)*absKpowZeta;
				if(j!=0){			
					fDIm_u[0](dimx-j,i)=fIm_u[0](dimx-j,i)*absKpowZeta;
					fDIm_u[1](dimx-j,i)=fIm_u[1](dimx-j,i)*absKpowZeta;
				}
				if(i!=0){			
					fDIm_u[0](j,dimy-i)=fIm_u[0](j,dimy-i)*absKpowZeta;
					fDIm_u[1](j,dimy-i)=fIm_u[1](j,dimy-i)*absKpowZeta;
				}
				if(i!=0&&j!=0){			
					fDIm_u[0](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*absKpowZeta;
					fDIm_u[1](dimx-j,dimy-i)=fIm_u[1](dimx-j,dimy-i)*absKpowZeta;
				}
			}	
			
		}
        //attention pas complexe conjuge en N/2
        for(int i=0;i<halfNby;i++){
            normK=0.25+(double)i/(double)dimy*(double)i/(double)dimy;
            absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
            fDIm_u[0](halfNbx,i)=fIm_u[0](halfNbx,i)*absKpowZeta;
            fDIm_u[1](halfNbx,i)=fIm_u[1](halfNbx,i)*absKpowZeta;
            if (i!=0){ 
                fDIm_u[0](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*absKpowZeta;
                fDIm_u[1](halfNbx,dimy-i)=fIm_u[1](halfNbx,dimy-i)*absKpowZeta;
            }
        }
        for(int j=0;j<halfNbx;j++){
			normK=0.25+(double)j/(double)dimx*(double)j/(double)dimx;
			absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
		    fDIm_u[0](j,halfNby)=fIm_u[0](j,halfNby)*absKpowZeta;
			fDIm_u[1](j,halfNby)=fIm_u[1](j,halfNby)*absKpowZeta;
            if (j!=0){ 
                fDIm_u[0](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*absKpowZeta;
                fDIm_u[1](dimx-j,halfNby)=fIm_u[1](dimx-j,halfNby)*absKpowZeta;
            }
        }
        normK=0.25+0.25;
        absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
        fDIm_u[0](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*absKpowZeta;
        fDIm_u[1](halfNbx,halfNby)=fIm_u[1](halfNbx,halfNby)*absKpowZeta;
        /**/ 
		
	}
    /**/ 
	//----Inverse FFT
	return (fDIm_u.get_FFT(true))[0];
	
}

CImg<double>  fracDerivOrInt_CurlDualAdjoint2Part(const CImg<double> &Im,double minusZeta){
	
	double zeta=-1*minusZeta;
	CImgList<double> fIm_u;
	CImgList<double> fIm_v;
    
	//----Direct FFT
	fIm_u=(Im.get_slice(0)).get_FFT(false);
	fIm_v=(Im.get_slice(1)).get_FFT(false);
	
	CImgList<double> fDIm_u(fIm_u);
	CImgList<double> fDIm_v(fIm_v);
	
	int dimx=Im.width();
	int dimy=Im.height();
	int halfNbx=dimx/2;
	int halfNby=dimy/2;
	double absKpowZeta;
	double normK;
	
	//----Fractional derivatiave/intergation
	if(zeta!=0){
		for(int i=0;i<halfNby;i++){
			for(int j=0;j<halfNbx;j++){
				if (i==0 && j==0){absKpowZeta=0;}
				else{
					normK=(double)j/(double)dimx*(double)j/(double)dimx+(double)i/(double)dimy*(double)i/(double)dimy;
					absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
				}	
				fDIm_u[0](j,i)=fIm_u[0](j,i)*absKpowZeta;
				fDIm_u[1](j,i)=fIm_u[1](j,i)*absKpowZeta;
				fDIm_v[0](j,i)=fIm_v[0](j,i)*absKpowZeta;
				fDIm_v[1](j,i)=fIm_v[1](j,i)*absKpowZeta;
				if(j!=0){			
					fDIm_u[0](dimx-j,i)=fIm_u[0](dimx-j,i)*absKpowZeta;
					fDIm_u[1](dimx-j,i)=fIm_u[1](dimx-j,i)*absKpowZeta;
					fDIm_v[0](dimx-j,i)=fIm_v[0](dimx-j,i)*absKpowZeta;
					fDIm_v[1](dimx-j,i)=fIm_v[1](dimx-j,i)*absKpowZeta;
				}
				if(i!=0){			
					fDIm_u[0](j,dimy-i)=fIm_u[0](j,dimy-i)*absKpowZeta;
					fDIm_u[1](j,dimy-i)=fIm_u[1](j,dimy-i)*absKpowZeta;
					fDIm_v[0](j,dimy-i)=fIm_v[0](j,dimy-i)*absKpowZeta;
					fDIm_v[1](j,dimy-i)=fIm_v[1](j,dimy-i)*absKpowZeta;
				}
				if(i!=0&&j!=0){			
					fDIm_u[0](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*absKpowZeta;
					fDIm_u[1](dimx-j,dimy-i)=fIm_u[1](dimx-j,dimy-i)*absKpowZeta;
					fDIm_v[0](dimx-j,dimy-i)=fIm_v[0](dimx-j,dimy-i)*absKpowZeta;
					fDIm_v[1](dimx-j,dimy-i)=fIm_v[1](dimx-j,dimy-i)*absKpowZeta;
				}
			}	
			
		}
        /**/
        //attention pas complexe conjuge en N/2
        for(int i=0;i<halfNby;i++){
            normK=0.25+(double)i/(double)dimy*(double)i/(double)dimy;
            absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
            fDIm_u[0](halfNbx,i)=fIm_u[0](halfNbx,i)*absKpowZeta;
            fDIm_u[1](halfNbx,i)=fIm_u[1](halfNbx,i)*absKpowZeta;
            fDIm_v[0](halfNbx,i)=fIm_v[0](halfNbx,i)*absKpowZeta;
            fDIm_v[1](halfNbx,i)=fIm_v[1](halfNbx,i)*absKpowZeta;
            if (i!=0){ 
                fDIm_u[0](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*absKpowZeta;
                fDIm_u[1](halfNbx,dimy-i)=fIm_u[1](halfNbx,dimy-i)*absKpowZeta;
                fDIm_v[0](halfNbx,dimy-i)=fIm_v[0](halfNbx,dimy-i)*absKpowZeta;
                fDIm_v[1](halfNbx,dimy-i)=fIm_v[1](halfNbx,dimy-i)*absKpowZeta;
            }
        }
        for(int j=0;j<halfNbx;j++){
			normK=0.25+(double)j/(double)dimx*(double)j/(double)dimx;
			absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
		    fDIm_u[0](j,halfNby)=fIm_u[0](j,halfNby)*absKpowZeta;
			fDIm_u[1](j,halfNby)=fIm_u[1](j,halfNby)*absKpowZeta;
			fDIm_v[0](j,halfNby)=fIm_v[0](j,halfNby)*absKpowZeta;
			fDIm_v[1](j,halfNby)=fIm_v[1](j,halfNby)*absKpowZeta;
            if (j!=0){ 
                fDIm_u[0](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*absKpowZeta;
                fDIm_u[1](dimx-j,halfNby)=fIm_u[1](dimx-j,halfNby)*absKpowZeta;
                fDIm_v[0](dimx-j,halfNby)=fIm_v[0](dimx-j,halfNby)*absKpowZeta;
                fDIm_v[1](dimx-j,halfNby)=fIm_v[1](dimx-j,halfNby)*absKpowZeta;
            }
        }
        normK=0.25+0.25;
        absKpowZeta=pow(2.*M_PI*sqrt(normK),zeta);
        fDIm_u[0](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*absKpowZeta;
        fDIm_u[1](halfNbx,halfNby)=fIm_u[1](halfNbx,halfNby)*absKpowZeta;
        fDIm_v[0](halfNbx,halfNby)=fIm_v[0](halfNbx,halfNby)*absKpowZeta;
        fDIm_v[1](halfNbx,halfNby)=fIm_v[1](halfNbx,halfNby)*absKpowZeta;
        /**/ 
		
		fIm_u=fDIm_u;
		fIm_v=fDIm_v;
		
	}
	//---- curl dual adjoint operator 
	double factorWav_u, factorWav_v;
	for(int i=0;i<halfNby;i++){
		for(int j=0;j<halfNbx;j++){
            
			if (i!=0){factorWav_u=-1./(2.*M_PI*(double)i/(double)dimy);}
			else {factorWav_u=0;}
			if (j!=0){factorWav_v=1./(2.*M_PI*(double)j/(double)dimx);}
			else {factorWav_v=0;}

            fDIm_u[0](j,i)=-1*fIm_u[1](j,i)*factorWav_u;
            fDIm_u[1](j,i)=fIm_u[0](j,i)*factorWav_u;
            fDIm_v[0](j,i)=-1*fIm_v[1](j,i)*factorWav_v;
            fDIm_v[1](j,i)=fIm_v[0](j,i)*factorWav_v;

			if(j!=0&& j!=halfNbx){			
				fDIm_u[0](dimx-j,i)=-1*fIm_u[1](dimx-j,i)*+1*factorWav_u;
				fDIm_u[1](dimx-j,i)=fIm_u[0](dimx-j,i)*+1*factorWav_u;
				fDIm_v[0](dimx-j,i)=-1*fIm_v[1](dimx-j,i)*-1*factorWav_v;
				fDIm_v[1](dimx-j,i)=fIm_v[0](dimx-j,i)*-1*factorWav_v;
			}
			if(i!=0 && i!=halfNby){			
				fDIm_u[0](j,dimy-i)=-1*fIm_u[1](j,dimy-i)*-1*factorWav_u;
				fDIm_u[1](j,dimy-i)=fIm_u[0](j,dimy-i)*-1*factorWav_u;
				fDIm_v[0](j,dimy-i)=-1*fIm_v[1](j,dimy-i)*+1*factorWav_v;
				fDIm_v[1](j,dimy-i)=fIm_v[0](j,dimy-i)*+1*factorWav_v;
			}
			if(i!=0&&i!=halfNby&&j!=0&& j!=halfNbx){			
				fDIm_u[0](dimx-j,dimy-i)=-1*fIm_u[1](dimx-j,dimy-i)*-1*factorWav_u;
				fDIm_u[1](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*-1*factorWav_u;
				fDIm_v[0](dimx-j,dimy-i)=-1*fIm_v[1](dimx-j,dimy-i)*-1*factorWav_v;
				fDIm_v[1](dimx-j,dimy-i)=fIm_v[0](dimx-j,dimy-i)*-1*factorWav_v;
			}
		}
	}	    
    //attention pas complexe conjuge en N/2
    for(int i=0;i<halfNby;i++){
        if (i!=0){
            factorWav_u=-1./(2.*M_PI*(double)i/(double)dimy);
        }
        else {factorWav_u=0;}
        factorWav_v=1./(2.*M_PI*(double)halfNbx/(double)dimx);
        
        fDIm_u[0](halfNbx,i)=-1*fIm_u[1](halfNbx,i)*factorWav_u;
        fDIm_u[1](halfNbx,i)=fIm_u[0](halfNbx,i)*factorWav_u;
        fDIm_v[0](halfNbx,i)=-1*fIm_u[1](halfNbx,i)*factorWav_v;
        fDIm_v[1](halfNbx,i)=fIm_u[0](halfNbx,i)*factorWav_v;	
        if (i!=0){ 
            fDIm_u[0](halfNbx,dimy-i)=-1*fIm_u[1](halfNbx,dimy-i)*-1*factorWav_u;
            fDIm_u[1](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*-1*factorWav_u;
            fDIm_v[0](halfNbx,dimy-i)=-1*fIm_u[1](halfNbx,dimy-i)*factorWav_v;
            fDIm_v[1](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*factorWav_v;	
        }
    }
    for(int j=0;j<halfNbx;j++){
        factorWav_u=-1./(2.*M_PI*(double)halfNby/(double)dimy);
        if (j!=0){
            factorWav_v=1./(2.*M_PI*(double)j/(double)dimx);;
        }
        else {factorWav_v=0;}
        
        fDIm_u[0](j,halfNby)=-1*fIm_u[1](j,halfNby)*factorWav_u;
        fDIm_u[1](j,halfNby)=fIm_u[0](j,halfNby)*factorWav_u;
        fDIm_v[0](j,halfNby)=-1*fIm_u[1](j,halfNby)*factorWav_v;
        fDIm_v[1](j,halfNby)=fIm_u[0](j,halfNby)*factorWav_v;	
        if (j!=0){ 
            fDIm_u[0](dimx-j,halfNby)=-1*fIm_u[1](dimx-j,halfNby)*factorWav_u;
            fDIm_u[1](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*factorWav_u;
            fDIm_v[0](dimx-j,halfNby)=-1*fIm_u[1](dimx-j,halfNby)*-1*factorWav_v;
            fDIm_v[1](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*-1*factorWav_v;	
        }       
    }
    factorWav_u=-1./(2.*M_PI*(double)halfNby/(double)dimy);
    factorWav_v=1./(2.*M_PI*(double)halfNbx/(double)dimx);
    fDIm_u[0](halfNbx,halfNby)=-1*fIm_u[1](halfNbx,halfNby)*factorWav_u;
    fDIm_u[1](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*factorWav_u;
    fDIm_v[0](halfNbx,halfNby)=-1*fIm_u[1](halfNbx,halfNby)*factorWav_v;
    fDIm_v[1](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*factorWav_v;	
     
    /**/ 
	//----Inverse FFT
	CImg<double> ImOut(Im,"xyzc",0.);
	ImOut.get_shared_plane(0)=(fDIm_u.get_FFT(true))[0];
    ImOut.get_shared_plane(1)=(fDIm_v.get_FFT(true))[0];
    //
    
	return ImOut;
	
}


CImg<double> normalizeFourierDivFreeAnisWav(const CImg<double> &in){

	CImg<double> c(in,0.);
	return c;
}
CImg<double> normalizeFourierDivFreeAnisWavV0(const CImg<double> &in){
    
	CImg<double> c(in,0.);
	return c;
}


CImg<double> normalizeFourierDivFreeAnisWavDual(const CImg<double> &in){
    
	CImg<double> c(in,0.); 
	c/=2;
    cimg_forX(c,p){c(0,p,1)=0;c(p,0,0)=0;}
	return (c.get_shared_plane(0)+c.get_shared_plane(1));
    
    
}

/**
 * clapack-based operator* for matrix-matrix multiplication
 */
CImg<double> operator*(const CImg<double>&A, const CImg<double> &B){
    
    
    if (A.width()!=B.height()){
        cout << "Incompatible dimensions for matrix-matrix product: A.width()=" << A.width() << ", B.height()=" << B.height() << "; crashing !" << endl;
        exit(3);
    }
    CImg<double> result(B.width(),A.height());
    cblas_dgemm(CblasRowMajor, CblasNoTrans, CblasNoTrans,
                A.height(), B.width(), A.width(),
                1., A.data(), A.height(),
                B.data(), B.height(),
                0., result.data(), result.height());
    return result;  
}
 
/**
 * butterworth filter
 **/

CImg<double>  butterworth(const CImg<double> &Im,double k0,double ratioSpec, int order){
	
	    
    CImgList<double> fIm_u;
	
    //----Direct FFT
    fIm_u=Im.get_FFT(false);
    CImgList<double> fDIm_u(fIm_u);
    fDIm_u[0].fill(0); fDIm_u[1].fill(0);
    
    int dimx=Im.width();
    int dimy=Im.height();
    int halfNbx=dimx/2;
    int halfNby=dimy/2;
    double filter;
    double normK;
	
    //----Fractional derivatiave/intergation
	
    for(int i=0;i<halfNby;i++){
		for(int j=0;j<halfNbx;j++){
            normK=(double)j/(double)dimx*(double)j/(double)dimx+(double)i/(double)dimy*(double)i/(double)dimy;
            filter=1./(sqrt(1.+(1./ratioSpec-1.)*pow(sqrt(normK)/(k0/(double)dimx),(double) 2.*order)));
			fDIm_u[0](j,i)=fIm_u[0](j,i)*filter;
			fDIm_u[1](j,i)=fIm_u[1](j,i)*filter;
			if(j!=0){
				fDIm_u[0](dimx-j,i)=fIm_u[0](dimx-j,i)*filter;
				fDIm_u[1](dimx-j,i)=fIm_u[1](dimx-j,i)*filter;
			}
			if(i!=0){
				fDIm_u[0](j,dimy-i)=fIm_u[0](j,dimy-i)*filter;
				fDIm_u[1](j,dimy-i)=fIm_u[1](j,dimy-i)*filter;
			}
			if(i!=0&&j!=0){
				fDIm_u[0](dimx-j,dimy-i)=fIm_u[0](dimx-j,dimy-i)*filter;
				fDIm_u[1](dimx-j,dimy-i)=fIm_u[1](dimx-j,dimy-i)*filter;
			}
		}
    }
    /**/
    //attention pas complexe conjuge en N/2
    for(int i=0;i<halfNby;i++){
		normK=0.25+(double)i/(double)dimy*(double)i/(double)dimy;
        filter=1./sqrt(1.+(1./ratioSpec-1.)*pow(sqrt(normK)/(k0/(double)dimx),(double) 2.*order));
        fDIm_u[0](halfNbx,i)=fIm_u[0](halfNbx,i)*filter;
		fDIm_u[1](halfNbx,i)=fIm_u[1](halfNbx,i)*filter;
		if (i!=0){
            fDIm_u[0](halfNbx,dimy-i)=fIm_u[0](halfNbx,dimy-i)*filter;
            fDIm_u[1](halfNbx,dimy-i)=fIm_u[1](halfNbx,dimy-i)*filter;
        }
    }
    for(int j=0;j<halfNbx;j++){
        normK=0.25+(double)j/(double)dimx*(double)j/(double)dimx;
        filter=1./sqrt(1.+(1./ratioSpec-1.)*pow(sqrt(normK)/(k0/(double)dimx),(double) 2.*order));
       fDIm_u[0](j,halfNby)=fIm_u[0](j,halfNby)*filter;
        fDIm_u[1](j,halfNby)=fIm_u[1](j,halfNby)*filter;
        if (j!=0){
            fDIm_u[0](dimx-j,halfNby)=fIm_u[0](dimx-j,halfNby)*filter;
            fDIm_u[1](dimx-j,halfNby)=fIm_u[1](dimx-j,halfNby)*filter;
        }
    }
    normK=0.25+0.25;
    filter=1./sqrt(1.+(1./ratioSpec-1.)*pow(sqrt(normK)/(k0/(double)dimx),(double) 2.*order));
    fDIm_u[0](halfNbx,halfNby)=fIm_u[0](halfNbx,halfNby)*filter;
    fDIm_u[1](halfNbx,halfNby)=fIm_u[1](halfNbx,halfNby)*filter;
    /**/
	//----Inverse FFT
	CImg<double> ImOut;
	ImOut=(fDIm_u.get_FFT(true))[0];
	return ImOut;
	
}




//=================
//-----------period
//=================
inline int period(int ind,
                          int max){
    return (ind<0)?((max + ind%max)%max):ind%max;
};



