/*
 *  OpFlowTools_visu.cpp
 *  
 *
 *  Copyright 2014 INRIA. All rights reserved.
 *
 */

#include <iostream>
#include "OpFlowTools_visu.h"

using namespace cimg_library;
using namespace std;

/**
 *  jet_LUT256
 */
CImg<double> jet_LUT256(){
  const int uh=32, th=96, uq=64;
  const double delta=255./63;
  CImg<double> jet(1,256,1,3, 0.);
  
  //-red channel
  for (int k=0; k<uq; k++) {jet(0,th+k,0) = k*delta;}
  for (int k=0; k<uq; k++) {jet(0,th+uq+k,0) = 255.;}
  for (int k=0; k<uh; k++) {jet(0,th+uq+uq+k,0) = 255.-k*delta;}
  //-green
  for (int k=0; k<uq; k++) {jet(0,uh+k,1) = k*delta;}
  for (int k=0; k<uq; k++) {jet(0,uh+uq+k,1) = 255.;}
  for (int k=0; k<uq; k++) {jet(0,uh+uq+uq+k,1) = 255.-k*delta;}
  //-blue
  for (int k=0; k<uh; k++) {jet(0,k,2) = (uh+k)*delta;}
  for (int k=0; k<uq; k++) {jet(0,uh+k,2) = 255.;}
  for (int k=0; k<uq; k++) {jet(0,uh+uq+k,2) = 255.-k*delta;}
  
  return jet;
};

/**
 *  gradient2_LUT256
 */
CImg<double> gradient2_LUT256(int r1,
                              int g1,
                              int b1,
                              int r2,
                              int g2,
                              int b2){

  CImg<double> gradient(1,256,1,3, 0.);
  double deltaR,deltaG,deltaB;
  
  deltaR = (r2-r1)/255.;
  deltaG = (g2-g1)/255.;
  deltaB = (b2-b1)/255.;
  
  for (int k=0; k<256; k++){
    gradient(0,k,0,0) = r1 + k*deltaR;
    gradient(0,k,0,1) = g1 + k*deltaG;
    gradient(0,k,0,2) = b1 + k*deltaB;
  }
  
  return gradient;
}

/**
 *  gradient3_LUT256
 */
CImg<double> gradient3_LUT256(int r1,
                              int g1,
                              int b1,
                              int r2,
                              int g2,
                              int b2,
                              int r3,
                              int g3,
                              int b3){
  
  CImg<double> gradient(1,256,1,3, 0.);
  double deltaR,deltaG,deltaB;
  
  deltaR = (r2-r1)/127.;
  deltaG = (g2-g1)/127.;
  deltaB = (b2-b1)/127.;
  
  for (int k=0; k<128; k++){
    gradient(0,k,0,0) = r1 + k*deltaR;
    gradient(0,k,0,1) = g1 + k*deltaG;
    gradient(0,k,0,2) = b1 + k*deltaB;
  }
  
  deltaR = (r3-r2)/127.;
  deltaG = (g3-g2)/127.;
  deltaB = (b3-b2)/127.;
  
  for (int k=0; k<128; k++){
    gradient(0,128+k,0,0) = r2 + k*deltaR;
    gradient(0,128+k,0,1) = g2 + k*deltaG;
    gradient(0,128+k,0,2) = b2 + k*deltaB;
  }
  
  return gradient;
}


/**
 *  hardLight
 */
CImg<double> hardLight(CImg<double>Image,
                       CImg<double>Mask){
  
  CImg<double> out(Image.width(),Image.height(),1,3, 0.);
  
  //-check sizes
  if (Image.is_sameXY(Mask)) {
    cimg_forXY(Mask, x,y){
      //-lighter areas of mask
      if (Mask(x,y)>128.) {
        //-each channel
        for (int c=0; c<3; c++) {out(x,y,c) = 255.-(255.-2.*(Mask(x,y)-128.))*(255.-Image(x,y,c))/256.;}
      }
      //-darker areas
      else {
        //-each channel
        for (int c=0; c<3; c++) {out(x,y,c) = 2.*Mask(x,y)*Image(x,y,c)/256.;}
      }
    }
  }
  else {
   cout << "Image and Mask (x,y) dimensions are not the same ! 0-filled image returned :(." << endl; 
  }
  
  return out;
};


/**
 *  motionColorCode
 */
CImg<double> motionColorCode(const CImg<double> &UV,
                             double normValue){

  CImg<double> HSVmotion(UV.width(),UV.height(),1,3,0.);
  double temp, pi2 = 2.*cimg::PI;
  
  //-Hue channel
  //-------------------
  //-get local angle between [0,2PI]
  cimg_forXY(UV,x,y){
    temp = atan2(UV(x,y,1),UV(x,y,0));
    HSVmotion(x,y,0) = (temp<0.)?temp+pi2:temp;
  };
  //-bring back to [0,360];
  HSVmotion.get_shared_channel(0) /= (pi2/360.);    
   
  //-Saturation channel
  //--------------
  //-compute norm
  HSVmotion.get_shared_channel(1) = (UV.get_shared_plane(0).get_sqr()+UV.get_shared_plane(1).get_sqr()).sqrt();
  //-normalize by max or specified value, if so
  temp = (normValue)?(normValue):(HSVmotion.get_shared_channel(1).max());
  cout << "Max velocity norm: " << HSVmotion.get_shared_channel(1).max() << endl ;
  cout << "Normalization value: " << temp << endl ;
  HSVmotion.get_shared_channel(1) /= temp;
  
  //-Value channel
  //-------------------
  HSVmotion.get_shared_channel(2) = 1.;  
  
  return HSVmotion.HSVtoRGB();
};