/*------------------------------------------------------------------------ */
/* FBm Divergence-Free Wavelet-based Optical-Flow main.cpp                 */
/*      Written By Patrick Héas.                                           */
/*      Copyright 2014 INRIA. All rights reserved                          */
/*------------------------------------------------------------------------ */

#include <iostream>
#include <fstream>
#include <string>
#include "aniWavOpticalFlow.h"

using namespace std;
using namespace cimg_library;

/*====================================
 ---------------MAIN-------------------
 ====================================*/

int main(int argc, char** argv){
    
    //--Arguments-------
    cout<<"INPUTS:\n (Minimal arguments are -i0 and -i1) "<<endl;
    //--filename
    string I0name = cimg_option("-i0","","nom Image 0.");
    string I1name = cimg_option("-i1","","nom Image 1.");
    string output = cimg_option("-out","./","output directory (and pattern).");
    string coeffDir = cimg_option("-coef","../vectorsNew/","input connection coeficient file directory.");
    //-regularization
    const bool frac  = cimg_option("-frac", false, "fractional (true) or divergence-free (false) wavelet basis.");
    double alph = cimg_option("-a",0.,"alpha value (regularization parameter).");
    const bool app = cimg_option("-app", false, "connection coef approx for regularization");
    double zet = cimg_option("-z",-3.,"power law exponent (zeta) for isotropic spectrum decay (Hurst exponent = -0.5*(zeta +1) ).\n\t\t\t\t\t   Possible value in {1, 1.02, 1.66667, 1.99, 2, 2.001, 2.33333, 2.98, 3, 3.66667, 3.99, 4, 4.98, 5, 7}\n");
 
    //--Optionals ---------------
    cout<<"OPTIONAL INPUTS: "<<endl;
    cout<<"____________   ground truth filename  ____________  "<<endl;
    string UVtname = cimg_option("-uvt","","ground truth UV field filename (.motion/.inr).");
    string UV0name = cimg_option("-uv0","","initialization UV field filename (.motion/.inr).");
    cout<<"____________  regularization type  ____________  "<<endl;
    const int reg = cimg_option("-r",6,"type of regularization (1: grad u, 2: 2nd order, 3: laplacian, 4: div, 5: grad curl, 6: div-free fBM prior).");
     cout<<"____________  cropping options  ____________  "<<endl;
    int cropx = cimg_option("-cropX",0,"Width of cropped area (0 = use original width).");
    int cropy = cimg_option("-cropY",0,"Height of cropped area (0 = use original height).");
    int offx = cimg_option("-offX",0,"Horizontal offset of cropped area.");
    int offy = cimg_option("-offY",0,"Vertical offset of cropped area.");
     cout<<"____________  initialisation  ____________  "<<endl;
    const double uMean = cimg_option("-uMean", 0., "Mean horizontal velocity (positive = rightward). Implicit -uv0.");
    const double vMean = cimg_option("-vMean", 0., "Mean vertical velocity (positive = downward). Implicit -uv0.");
    const int nI = cimg_option("-nI",0," number of initialized scales (0: coarse, >0: coarse + details). Only with -uv0 specified.");
     cout<<"____________  fBm and image synthesis (put same image to deform in arguments -i0 and -i1) ____________  "<<endl;
    double fac = cimg_option("-fac",0.,"if !=0 then energy factor for synthetic image generation by fBm process.");
    double sigNoise = cimg_option("-sigN",0.,"additive Gaussian noise stdev on generated synthetic images.");
     cout<<"____________   optimization  ____________  "<<endl;
    const lbfgsfloatval_t eps = cimg_option("-eps",1e-4,"minimum decrease of objective function for lbfgs.");
     cout<<"____________   visualization  ____________  "<<endl;
    const bool disp = cimg_option("-disp", false, "display input (I0, I1, Mask) & output (U, V) images.");
    cout<<"____________   others  ____________  "<<endl;
    const int nE = cimg_option("-nE",-1,"number of estimated scales (0: coarse, >0: coarse + details).");
    const int nM = cimg_option("-nM",10,"number of vanishing moments (must be EVEN, because of coiflets !!).");
    const bool div0 = cimg_option("-div0", true, "Divergence free constraint.");
    const double nMinPwr = cimg_option("-pwrMin",0.,"scale where fBm regularization begins (1 <= pwrMin <= nEstim).");
    const double nMaxPwr = cimg_option("-pwrMax",1.e6,"scale where fBm regularization end (pwrMin <= pwrMax <= nEstim).");
    const bool period = cimg_option("-bc", true, "boundary condition : 0 = Neumann, 1 = periodic.");
    const double sigma = cimg_option("-s",0.,"image data pre-smoothing parameter value.");

    //***********************************
    //***********************************
    //      input files & parameters
    //***********************************
    //***********************************
 
    //unnecessary parameter, to be removed ...
    const bool quiv =  false;double dim = 2.;double tau = 0;
    CImg<double> Bgin, Maskin, Mask;
    
    //Declaration of some variables
    CImg<double> I0in, I1in, UV0in, UVTin, I0, I1, UV0, UVestim, UVtrue;
    int err = 0;
    int Ox=0, Oy=0;
    size_t dotpos;
    unsigned long clock;
    double zetIn=zet;
    zet=(1.-dim+zet)/2.;
    if (frac || reg==6 || fac !=0.){
        cout<<"\nFourier decay exponent = "<<zetIn<<endl;
        cout<<"Fractional derivative order = "<<zet<<endl;
        cout<<"Hurst exponent = "<<-1*zet-1<<"\n"<<endl;
    }
    
    //---------input images
    cout << "Input images:" << endl << "\t" << I0name << endl << "\t" << I1name << endl;
    I0in.assign(const_cast<char*>(I0name.c_str()));
    I1in.assign(const_cast<char*>(I1name.c_str()));
    I0in.channel(0);
    I1in.channel(0);
    //---------truth UV
    if (UVtname.size()>0) {UVTin.assign(loadMotion(const_cast<char*>(UVtname.c_str())));}
    //---------initialization UV
    if (UV0name.size()>0) {UV0in.assign(loadMotion(const_cast<char*>(UV0name.c_str())));}
    //-if -uMean or -vMean, given, override -uv0 and (re)set nI to 0
    if ((uMean!=0.)||(vMean!=0.)) {
        UV0in.assign(I0in.width(),I0in.height(),2,1);
        UV0in.get_shared_plane(0).fill(uMean);
        UV0in.get_shared_plane(1).fill(vMean);
    }
    //--crop images and optional motions if requested by user
    //-/!\ cropped images will be considered as input
    //-i.e. resulting motion field will have dimensions of cropped image
    if (cropx||cropy) {
        //-if one dimensions is ommited, correct
        if (cropx==0) { cropx = I0in.width(); }
        if (cropy==0) { cropy = I0in.height(); }
        I0in.crop(offx,offy,offx+cropx-1,offy+cropy-1);
        I1in.crop(offx,offy,offx+cropx-1,offy+cropy-1);
        if (!UV0in.is_empty()) { UV0in.crop(offx,offy,offx+cropx-1,offy+cropy-1); }
        if (!UVTin.is_empty()) { UVTin.crop(offx,offy,offx+cropx-1,offy+cropy-1); }
    }
    
    //*******************************
    //*******************************
    //  execution
    //*******************************
    //*******************************
    
    //-prepare images
    WavOpticalFlow::prepareInputImg(I0in,I1in,Bgin,Maskin,UV0in,UVTin,I0,I1,Mask,UV0,UVtrue,Ox,Oy);
    const int nD = (int)log2(I0.width()); // Depth of wavelet decomposition.
    int nEstim;
    (nE==-1)? nEstim = (int)log2(I0.width()): nEstim = nE; // Depth of wavelet estimation d.
    //-declare problem
    WavOpticalFlow *wOP;
    if (frac||div0){wOP=new WavOpticalFlow(I0,I1,Mask,UVtrue,nD,nEstim,nI,nM,1,sigma,reg,alph,zet,zetIn,frac,div0,period,nMinPwr,nMaxPwr,1,tau,app,coeffDir);  }
    else{wOP=new WavOpticalFlow(I0,I1,Mask,nD,nEstim,nI,nM,1,sigma,reg,alph);}
    if (fac!=0){
        CImgList<double> UVtrueI0I1;
        UVtrueI0I1=(*wOP).deformAndReturnImByUVFracDivFree(false,zet,fac,0,1000,sigNoise);
        UVtrue=UVtrueI0I1[0];I0=UVtrueI0I1[1];I1=UVtrueI0I1[2];
        UVtrue=UVtrueI0I1[0];I0=UVtrueI0I1[1];I1=UVtrueI0I1[2];
        saveMotion(const_cast<char*>((output + "SynthesisFBM_UVtrue.motion").c_str()), UVtrue);
        saveMotion(const_cast<char*>((output + "SynthesisFBM_UVtrue.inr").c_str()), UVtrue);
        I1.get_normalize(0,255).save_png(const_cast<char*>((output + "SynthesisFBM_Image1.png").c_str()));I0.get_normalize(0,255).save_png(const_cast<char*>((output + "SynthesisFBM_Image0.png").c_str()));
        int err = 0;
        double N=UVtrue.width();
        write_Spectrum(const_cast<char*>((output + "SynthesisFBM_UVspectrum.txt").c_str()),isotropicPSD(UVtrue,0.,2.*M_PI/N,0.1).width(),isotropicPSD(UVtrue,0.,2.*M_PI/N,0.1).get_shared_plane(0).data(),err);



    }
    
    if (disp) if (!UVtrue.is_empty()){
        CImgList<double>(I0,I1,UVtrue.get_shared_plane(0),UVtrue.get_shared_plane(1),curlFFT(UVtrue),true).display("Data");
    }
    else {CImgList<double>(I0,I1,true).display("Data");}
    //-launch time
    clock = cimg::time();
    //-print info
    (*wOP).info();
    //--solve
    //----------------
    if (!frac && !div0){
        cout<<"standard aniwavelets\n\n"<<endl;
        UVestim = (*wOP).solve(eps,UV0,0,0);
    }
    else{
        cout<<"div free / leray-projected fractional aniwavelets \n\n"<<endl;
        UVestim = (*wOP).solveFracDivFree(eps,UV0);
    }
 
    //*******************************
    //*******************************
    //  output
    //*******************************
    //*******************************

    if(disp) {
            CImgList<double>(UVestim.get_shared_plane(0),UVestim.get_shared_plane(1),true).display();
            CImg<double>(curlSpline(UVestim,sigma)).display("Vorticity");
    }
    (*wOP).output(output,Ox,Oy,I0in.width(),I0in.height());
    if (!UVtrue.is_empty()){
        (*wOP).compare(UVtrue,disp);
    }
    
    cout << "Total execution time : " << (cimg::time()-clock)/1000. << " s." << endl;
    cout << "****************************************************" << endl << endl;
    return err;
}

