/*
 *  specSlope.cpp
 *  
 *
 *  Created by Patrick Héas.
 *  Copyright 2014 INRIA. All rights reserved.
 *
 */

#include <iostream>
#include <fstream>
#include <string>
#include "CImg.h"
#include "OpFlowTools.h"
using namespace std;
using namespace cimg_library;

/*====================================
 ---------------MAIN-------------------
 ====================================*/
/*************************************************************************************************************/
double  beta_LeastSquareEstimator(const CImg<double> &S){
    
	double nom,den;
	double tmp1,tmp2,tmp3,tmp4; 
	tmp1=0;tmp2=0;tmp3=0;tmp4=0;
	double freq;
	for (int l=1; l<S.width();l++){ 
        freq=(S(l,0));
		tmp1+=log(freq)*log(freq);
		tmp2+=log(S(l,1))*log(freq);
		tmp3+=log(freq);
		tmp4+=log(S(l,1));
	}
	nom=tmp4*tmp1-tmp3*tmp2;
	den=(S.width()-1)*tmp1-tmp3*tmp3;
	return  exp(nom/den);
}


/*************************************************************************************************************/
double  power_LeastSquareEstimator(const CImg<double> &S){
    
	double nom,den;
	double tmp1,tmp2,tmp3,tmp4; 
	tmp1=0;tmp4=0;tmp3=0;tmp2=0;
	double freq;
	for (int l=1; l<S.width();l++){ 
        freq=(S(l,0));
		tmp1+=log(freq)*log(freq);
		tmp2+=log(freq)*log(S(l,1));
		tmp3+=log(freq);
		tmp4+=log(S(l,1));
	}
	nom=(S.width()-1)*tmp2-tmp4*tmp3;
	den=(S.width()-1)*tmp1-tmp3*tmp3;
	return  nom/den;
}

CImg<double> treshold_minmax(const CImg<double> &in, double treshmin, double treshmax, double flag){
    
	short int nb_col=in.width();
	short int nb_lig=in.height();
	CImg<double> out(nb_col,nb_lig);
	out=in;
	for (int i=0;i<nb_lig;i++){
        for (int j=0;j<nb_col;j++){
            if (out(j,i)<=treshmin||out(j,i)>=treshmax){
				out(j,i)=flag;
            }
        }
	}
	return out;	
	
}

int main(int argc, char** argv){
  
  //--options
  //-----------------
    const string UVname = cimg_option("-uv","","input file (scalar or vector)");
    const int mode = cimg_option("-m",1,"type de fichier en entree (0: .motion, 1: other motion file, 2: png or other scalar files)");
    const string UVtname = cimg_option("-uvt","","true file (scalar or vector)");
    const string outDir = cimg_option("-out","","output directory");
    const bool vorticity = cimg_option("-vort","false","vorticity visulaization");
    const double tresh = cimg_option("-tresh",0.25,"vorticity absolute treshold");
    const double Kmin = cimg_option("-kmin",10,"low frequency bound for MPSE calculus");
    const double smoothFac = cimg_option("-smooth",3.,"spectrum smoothing factor");
  int err;
    double slope,slopeRef,factor,factorRef,intDiffTmp=0,intDiff=0;
    CImg<double> UV;CImg<double> UVt;CImg<double> spectrum;CImg<double> tmpIm,tmpUv;
  
  //-lecture image
  //--------------
    cout << UVname <<endl;
    if (mode<2){
        (mode==1)? UV.assign(loadMotion(const_cast<char*>(UVname.c_str()))):UV.assign(read_MotionFile(const_cast<char*>(UVname.c_str()),err));
    }
    else{
        tmpUv.assign(loadIm(const_cast<char*>(UVname.c_str())));
        UV.assign(tmpUv.width(),tmpUv.height(),2,1,0.);UV.get_shared_plane(0)=tmpUv;UV.get_shared_plane(1)=tmpUv;
    }
        //UV.print();
    if (UVtname.size()>0){
        cout << UVtname <<endl;
        if (mode<2){
            (mode==1)? UVt.assign(loadMotion(const_cast<char*>(UVtname.c_str()))):UVt.assign(read_MotionFile(const_cast<char*>(UVtname.c_str()),err));
        }
        else{
            tmpUv.assign(loadIm(const_cast<char*>(UVtname.c_str())));
            UVt.assign(tmpUv.width(),tmpUv.height(),2,1,0.);UVt.get_shared_plane(0)=tmpUv;UVt.get_shared_plane(1)=tmpUv;
        }
       // UVt.print();
    }
    if (UV.depth()==1){};
  //----compute
    spectrum=isotropicPSD(UV,0.,2.*M_PI/UV.width(),0.1);
    slope = power_LeastSquareEstimator(spectrum);
    factor=log(beta_LeastSquareEstimator(spectrum));
    if (UVtname.size()>0){
        spectrum=isotropicPSD(UVt,0.,2.*M_PI/UV.width(),0.1);
        slopeRef = power_LeastSquareEstimator(spectrum);
        factorRef=log(beta_LeastSquareEstimator(spectrum));
        double logKmax=log(spectrum(spectrum.width()-1,0));
        double logKmin=log(Kmin);
        double a=(slope-slopeRef),b=(factor-factorRef);
        if (a!=0 && -1*b/a > 0 && -1*b/a < logKmax){intDiff=abs(a*logKmax+b)*(logKmax+b/a)/2-abs(b)/2*b/a;}
        else{intDiff=abs((a*logKmax/2+b)*logKmax);}
        if (logKmin>0){
            if (a!=0 && -1*b/a > 0 && -1*b/a < logKmin){intDiffTmp=abs(a*logKmin+b)*(logKmin+b/a)/2-abs(b)/2*b/a;}
            else{intDiffTmp=abs((a*logKmin/2+b)*logKmin);}
            intDiff-=intDiffTmp;
            if (intDiff<0){cout<<"!!!!!!!!!!!!!!!!!!!!!!!!!!!"<<intDiffTmp+intDiff<<"-"<<intDiffTmp<<endl;}
        }
        intDiff/=(logKmax-logKmin);

      }
    if (vorticity){
        tmpIm=treshold_minmax(curlSpline(UV,0), -1*tresh, 1.e10, -1*tresh);
        tmpIm=treshold_minmax(tmpIm, -1e10, tresh, tresh);
        ((tmpIm+tresh)*2*255).save(const_cast<char*>((outDir + "vortEstim.png").c_str()));
        if (UVtname.size()>0){
            tmpIm=treshold_minmax(curlSpline(UVt,0), -1*tresh, 1e10, -1*tresh);
            tmpIm=treshold_minmax(tmpIm, -1.e10, tresh, tresh);
            ((tmpIm+tresh)*2*255).save(const_cast<char*>((outDir + "vortTrue.png").c_str()));}
    } 
    
  //----write and free
  fstream fichier(const_cast<char*>((outDir + "spectrumSlopeFactor.txt").c_str()), ios::out | ios::trunc); 

  fichier << "slope .. factor .. intDiff: "<< slope << " .. " << factor << " .. "<< intDiff<< endl;    
  fichier.close();
    
    spectrum=isotropicPSD(UV,smoothFac,2.*M_PI/UV.width(),0.1);
    write_Spectrum(const_cast<char*>((outDir + "spectrum_UVestim.txt").c_str()),spectrum.width(),
                   spectrum.get_shared_plane(0).data(),err);
    if (UVtname.size()>0){
        spectrum=isotropicPSD(UVt,smoothFac,2.*M_PI/UVt.width(),0.1);
        write_Spectrum(const_cast<char*>((outDir + "spectrum_UVtrue.txt").c_str()),spectrum.width(),
                       spectrum.get_shared_plane(0).data(),err);
        spectrum=isotropicPSD(UVt-UV,smoothFac,2.*M_PI/UVt.width(),0.1);
        write_Spectrum(const_cast<char*>((outDir + "spectrum_UVerror.txt").c_str()),spectrum.width(),
                   spectrum.get_shared_plane(0).data(),err);
    }

  //-end
  //-----
  return 0;
}